'$FORM Scsinf
'$FORM Horton
'$FORM Nodelink
'$FORM Nodedef
'$FORM Ponddsn
'$FORM Rfllprop
'$FORM Rflledit
'$FORM Watrshed


Const MAXNODES = 8
Const MAXUNIT = 100
Const MAXSTEPS = 260
Const SSD_RELATIONSHIPS = 20

Const OSCREEN = 0
Const OPRINTER = 1
Const OFILE = 2

Const VNOTCH60 = 0
Const VNOTCH90 = 1
Const SCRECT = 2
Const BCRECT = 3
Const ORIFICE = 4

'Infiltration Types

Const SCS_INFILTRATION = 100
Const HORTON_INFILTRATION = 101

' Types of Rainfall

Const USER_RAINFALL = 100
Const SCSTYPEII_RAINFALL = 101
Const SCSTYPEIIFL_RAINFALL = 102
Const SCSTYPEIII_RAINFALL = 103
Const CONSTANT_RAINFALL = 104

Type WatershedType
        FileName                    As String * 100
        TotalArea                   As Single
        Label                       As String * 30
        InfMeth                     As Integer
        TimeCon                     As Single
        ImpArea                     As Single
        DCIA                        As Single
        AddAbsImp                   As Single
        AddAbsPerv                  As Single
        MaxInf                      As Single
        HIC                         As Single
        HIO                         As Single
        HK                          As Single
        CN                          As Single
        InitAbs                     As Single
        FracDCIA                    As Single
        FracNDCIA                   As Single
        FracPerv                    As Single
        SmoothFlag                  As Single
End Type

Type WeirType
        Label                           As String * 35
        Equation                        As String * 25
        Type                            As Integer
        NumberSSD                       As Integer
        Invert                          As Single
        Diameter                        As Single
        Width                           As Single
        C                               As Single
        n                               As Single
        
        Stage(SSD_RELATIONSHIPS)        As Single
        Discharge(SSD_RELATIONSHIPS)    As Single
End Type

Type PondType
        FileName                        As String * 100
                                                    ' *** Weir Parameters
        cw                              As Integer
        NumberWeirs                     As Integer
        Weir(2)                         As WeirType

        peakstep                        As Integer
        
        minstage                        As Single
        OrigVolume                      As Single
        MinStorage                      As Single
        SurfArea                        As Single
                                                    ' *** Routing Parameters
        NumberSteps                     As Integer
        TimeStep                        As Single
        InitStage                       As Single
        MaxStage                        As Single
        MaxVolume                       As Single
        PeakInflow                      As Single
        PeakOutflow                     As Single
        pstage(MAXSTEPS)                As Single
        inflow(MAXSTEPS)                As Single
        outflow(MAXSTEPS)               As Single
                                                    ' *** SSD Parameters
        NumberSSD                       As Integer
        Stage(SSD_RELATIONSHIPS)        As Single
        Discharge(SSD_RELATIONSHIPS)    As Single
        Storage(SSD_RELATIONSHIPS)      As Single
        srfarea(SSD_RELATIONSHIPS)      As Single
End Type

Type rainfalltype
        FileName                        As String * 100
        Type                            As Integer
        NumberSteps                     As Integer
        TimeStep                        As Integer
        Duration                        As Single
        Total                           As Single
        Rain(MAXSTEPS)                  As Single
End Type

Type UnitHydrographType
        FileName                        As String * 100
        NumberSteps                     As Integer
        Q(MAXUNIT)                      As Single
End Type

Type HydrographType
        FileName                        As String * 100
        hydrotype                       As String * 26
        Runoff(MAXSTEPS)                As Single
        infiltration(MAXSTEPS)          As Single
        outflow(MAXSTEPS)               As Single
        TimeCon                         As Single
        TimeStep                        As Single
        RatCoeff                        As Single
        KR                              As Single
        PeakAttFact                     As Single
        HydroSteps                      As Integer
        TotalExcess                     As Single
        TotalRunoff                     As Single
        TotalInfiltration               As Single
        TotalOutflow                    As Single
        PeakOutflow                     As Single
        TotalArea                       As Single
End Type

Global ODest  As String
Global OType As Integer

Global r     As rainfalltype
Global w     As WatershedType
Global h     As HydrographType
Global p     As PondType

Sub CalculatePondDischarge (Pond As PondType)
Dim i       As Integer
Dim j       As Integer

For i = 1 To Pond.NumberSSD
      Pond.Discharge(i) = 0
Next i

For i = 1 To Pond.NumberWeirs
      Pond.Weir(i).NumberSSD = Pond.NumberSSD
      For j = 1 To Pond.NumberSSD
            Pond.Weir(i).Stage(j) = Pond.Stage(j)
      Next j
            CalculateWeirDischarge Pond.Weir(i)
Next i

For i = 1 To Pond.NumberWeirs
      For j = 1 To Pond.NumberSSD
            Pond.Discharge(j) = Pond.Discharge(j) + Pond.Weir(i).Discharge(j)
      Next j
Next i


End Sub

Sub CalculateWeirDischarge (w As WeirType)

For j = 1 To w.NumberSSD
      height = w.Stage(j) - w.Invert
      If height < 0 Then height = 0

      Select Case w.Type
                Case VNOTCH60 To VNOTCH90   ' 60 Degree V-Notch
                      w.Discharge(j) = w.C * (height ^ w.n)
                Case SCRECT To BCRECT       'Rectangular Weir
                      w.Discharge(j) = w.C * (height ^ w.n)
                Case 5  'Orifice
                      If (height > w.Diameter) Then
                                height = 2 * 32.2 * (height - w.Diameter / 2)
                                w.Discharge(j) = w.C * (height ^ w.n)
                      Else
                                w.Discharge(j) = 2.5 * (height ^ 2.5)
                      End If
      End Select
Next j
End Sub

Sub GenerateHydrograph (HydType As Integer)
        If r.NumberSteps = 0 Then
              MsgBox ("There is No Rainfall File")
              Exit Sub
        End If

        If w.TotalArea = 0 Or w.InfMeth = 0 Then
              MsgBox ("There is No Watershed File")
              Exit Sub
        End If

        GenerateInstantHydrograph w, r, h

Select Case HydType
        Case 1:  SantaBarbara h
        Case 2:  SCSMethod 0, h
        Case 3:  SCSMethod 484, h
        Case 4:  SCSMethod 256, h
End Select
      MsgBox (RTrim$(h.hydrotype) + " Hydrograph Generated")
End Sub

Sub GenerateInstantHydrograph (TW As WatershedType, TR As rainfalltype, TH As HydrographType)

Dim i                                   As Integer
Dim j                                   As Integer
Dim PotentialInfiltration               As Single
Dim TotalPerviousRainfall               As Single
Dim CumulativeInfiltration              As Single
Dim CumulativePerviousAbstraction       As Single
Dim CumulativeImperviousAbstraction     As Single
Dim CumulativePerviousExcess            As Single
Dim LastCumulativePerviousRunoff        As Single
Dim CumulativePerviousRunoff            As Single
Dim ImperviousExcess                    As Single
Dim PerviousExcess                      As Single
Dim ImpervioustoPerviousExcess          As Single
Dim DummyCumInf                         As Single
Dim PerviousRunoff                      As Single
Dim ImperviousRunoff                    As Single
Dim TempExcess                          As Single

NewHydrograph TH

CumulativePerviousExcess = 0
CumulativePerviousRunoff = 0
CumulativeInfiltration = 0
CumulativeImperviousAbstraction = 0
ImperviousExcess = 0
TR.Total = 0
TotalPerviousRainfall = 0
PerviousExcess = 0
LastCumulativePerviousRunoff = 0

TH.TimeCon = TW.TimeCon
TH.TimeStep = TR.TimeStep
TH.HydroSteps = TR.NumberSteps
TH.TotalArea = TW.TotalArea

For i = 1 To TR.NumberSteps
      TR.Total = TR.Total + TR.Rain(i)
      Time = i * TR.TimeStep

  '       **  Routing of Abstraction on Impervious Surface **

      If TW.AddAbsImp > TR.Total Then
                ImperviousExcess = 0
                CumulativeImperviousAbstraction = CumulativeImperviousAbstraction + TR.Rain(i)

      ElseIf TR.Total - TR.Rain(i) < TW.AddAbsImp And TR.Total > TW.AddAbsImp Then
                ImperviousExcess = TR.Rain(i) - (TW.AddAbsImp - CumulativeImperviousAbstraction)
                CumulativeImperviousAbstraction = TW.AddAbsImp
                If ImperviousExcess < 0 Then ImperviousExcess = 0

      Else
                ImperviousExcess = TR.Rain(i)
                If ImperviousExcess < 0 Then ImperviousExcess = 0
      End If

      ImpervioustoPerviousExcess = TW.FracNDCIA * ImperviousExcess * TW.TotalArea / (TW.TotalArea - TW.ImpArea)
      ImperviousRunoff = ImperviousExcess
      PerviousRainfall = TR.Rain(i) + ImpervioustoPerviousExcess
      TotalPerviousRainfall = PerviousRainfall + TotalPerviousRainfall

  '       **      Routing of Abstraction on Pervious Surface      **

      If TW.AddAbsPerv > TotalPerviousRainfall Then
                PerviousExcess = 0
                CumulativePerviousAbstraction = TotalPerviousRainfall
      Else
                PerviousExcess = PerviousRainfall + (CumulativePerviousAbstraction - TW.AddAbsPerv)
                CumulativePerviousAbstraction = TW.AddAbsPerv
      End If

      CumulativePerviousExcess = CumulativePerviousExcess + PerviousExcess

      If TW.InfMeth = HORTON_INFILTRATION Then
                ExitFlag = False
                j = 0
                While Not ExitFlag
                                DummyCumInf = (TW.HIC * j * TR.TimeStep) / 60! + (TW.HIO - TW.HIC) * (1 - Exp((TW.HK * j * TR.TimeStep / 60))) / TW.HK
                                If CumulativeInfiltration > DummyCumInf Then
                                                j = j + 1
                                Else
                                                DummyTime = j * TR.TimeStep / 60!
                                                ExitFlag = True
                                End If
                Wend
                PotentialInfiltration = TW.HIC + (TW.HIO - TW.HIC) * Exp((TW.HK * DummyTime)) * TR.TimeStep / 60

                If CumulativeInfiltration < TW.MaxInf Then
                                If PerviousExcess > PotentialInfiltration Then
                                                PerviousRunoff = PerviousExcess - PotentialInfiltration
                                                TH.infiltration(i) = PotentialInfiltration
                                Else
                                                PerviousRunoff = 0
                                                TH.infiltration(i) = PerviousExcess
                                End If
                Else
                                PerviousRunoff = PerviousExcess
                                TH.infiltration(i) = 0
                End If
      End If

      If TW.InfMeth = SCS_INFILTRATION Then
                SPrime = 1000 / TW.CN - 10
                AbsFact = TW.InitAbs * SPrime
                If CumulativePerviousExcess > AbsFact Then
                                CumulativePerviousRunoff = ((CumulativePerviousExcess - AbsFact) ^ 2) / (CumulativePerviousExcess + (1 - TW.InitAbs) * SPrime)
                                PerviousRunoff = CumulativePerviousRunoff - LastCumulativePerviousRunoff
                                LastCumulativePerviousRunoff = CumulativePerviousRunoff
                                If CumulativeInfiltration < TW.MaxInf Then
                                                TH.infiltration(i) = PerviousExcess - PerviousRunoff
                                Else
                                                TH.infiltration(i) = 0
                                                PerviousRunoff = PerviousExcess
                                End If
                Else
                                If CumulativeInfiltration < TW.MaxInf Then
                                                PerviousRunoff = 0
                                                TH.infiltration(i) = PerviousExcess
                                Else
                                                PerviousRunoff = PerviousExcess
                                                TH.infiltration(i) = 0
                                End If
                End If
      End If

      CumulativeInfiltration = CumulativeInfiltration + TH.infiltration(i)
      TH.infiltration(i) = TH.infiltration(i) * TW.FracPerv
      TempExcess = TW.FracDCIA * ImperviousRunoff + TW.FracPerv * PerviousRunoff
      TH.TotalExcess = TempExcess + TH.TotalExcess
      TH.Runoff(i) = 60 * TempExcess * TW.TotalArea * 1.008 / TR.TimeStep
      TH.TotalInfiltration = TH.TotalInfiltration + TH.infiltration(i)
Next i
End Sub

Function getVal! (n As Integer, xo As Single, x() As Single, y() As Single)

i = n
                While xo < x(i) And i > 0
                                i = i - 1
                Wend

                If i < n Then
                                If x(i + 1) <> x(i) Then
                                                m! = (y(i + 1) - y(i)) / (x(i + 1) - x(i))
                                Else
                                                m! = 0
                                End If

                Else
                                If x(i - 1) <> x(i) Then
                                                m! = (y(i) - y(i - 1)) / (x(i) - x(i - 1))
                                Else
                                                m! = 0
                                End If

                End If
                b! = y(i) - m! * x(i)
                getVal! = m! * xo + b!

End Function

Sub MakeDimRainfall (r As rainfalltype)
'REDIM a$(5)
Dim i                   As Integer
Dim peakstep            As Integer
Dim a                   As Single
Dim PeakPrecip          As Single
Dim DimTime             As Single
Dim RainThisStep        As Single
Dim RainLastStep        As Single
Dim TotalPrecip         As Single
Dim Precip              As Single
Dim ActualPrecip        As Single


StartofMakeDimRainFall:

'IF disttype <> 4 THEN
  '        CreateRainFile r
'ELSE
  '        r.RainTimeStep = 30
  '        r.RainDuration = 24
  '        r.NumRainSteps = r.RainDuration * 60 / r.RainTimeStep + .5
'END IF

RainLastStep = 0
'LOCATE 2, 2
RainLastStep = 0
TotalPrecip = 0
For i = 1 To r.NumberSteps
            DimTime = i / r.NumberSteps
            Select Case r.Type
                  Case SCSTYPEII_RAINFALL
                                If DimTime <= 1 Then Precip = .0178 + 1.893 * DimTime - .9192 * DimTime ^ 2
                                If DimTime = .5 Then Precip = .663
                                If DimTime < .5 Then Precip = 22.394 - 101.8 * DimTime + 116.5 * DimTime ^ 2
                                If DimTime < .44 Then Precip = .4395 * DimTime ^ 1.195
                                If DimTime < .0271 Then Precip = .226 * DimTime

                  Case SCSTYPEIIFL_RAINFALL
                                If DimTime <= 1 Then Precip = -.0508 + 2.0277 * DimTime - .9855 * DimTime ^ 2
                                If DimTime = .5 Then Precip = .607
                                If DimTime < .5 Then Precip = .00953 + .0726 * DimTime + .9643 * DimTime ^ 2
                                If DimTime < .0271 Then Precip = .226 * DimTime

                  Case SCSTYPEIII_RAINFALL
                                If DimTime <= 1 Then Precip = 1.003 * DimTime ^ .2657
                                If DimTime < .7 Then Precip = -3.8077 + 13.7517 * DimTime - 10.009 * DimTime ^ 2
                                If DimTime = .5 Then Precip = .5
                                If DimTime <= .5 Then Precip = .01 * Exp(7.212 * DimTime)
                                If DimTime < .0271 Then Precip = .224 * DimTime

                Case CONSTANT_RAINFALL
                                Precip = i / r.NumberSteps
                'IF disttype = 4 THEN
                  '        SELECT CASE i
                  '                CASE 1: Precip = .0013: CASE 2: Precip = .0027: CASE 3: Precip = .004
                  '                CASE 4: Precip = .008: CASE 5: Precip = .013: CASE 6: Precip = .021
                  '                CASE 7: Precip = .034: CASE 8: Precip = .049: CASE 9: Precip = .07
                  '                CASE 10: Precip = .091: CASE 11: Precip = .113: CASE 12: Precip = .136
                  '                CASE 13: Precip = .183: CASE 14: Precip = .236: CASE 15: Precip = .304
                  '                CASE 16: Precip = .386: CASE 17: Precip = .49: CASE 18: Precip = .593
                  '                CASE 19: Precip = .666: CASE 20: Precip = .736: CASE 21: Precip = .772
                  '                CASE 22: Precip = .809: CASE 23: Precip = .832: CASE 24: Precip = .854
                  '                CASE 25: Precip = .876: CASE 26: Precip = .898: CASE 27: Precip = .913
                  '                CASE 28: Precip = .927: CASE 29: Precip = .933: CASE 30: Precip = .939
                  '                CASE 31: Precip = .944: CASE 32: Precip = .949: CASE 33: Precip = .953
                  '                CASE 34: Precip = .958: CASE 35: Precip = .963: CASE 36: Precip = .968
                  '                CASE 37: Precip = .972: CASE 38: Precip = .976: CASE 39: Precip = .979
                  '                CASE 40: Precip = .983: CASE 41: Precip = .985: CASE 42: Precip = .988
                  '                CASE 43: Precip = .99: CASE 44: Precip = .993: CASE 45: Precip = .9955
                  '                CASE 46: Precip = .998: CASE 47: Precip = .999: CASE 48: Precip = 1!
                  '        END SELECT
                'END IF
            End Select
            RainThisStep = Precip * r.Total
            r.Rain(i) = (RainThisStep - RainLastStep)
            If r.Rain(i) < .001 Then r.Rain(i) = .001
            r.Rain(i) = Int(r.Rain(i) * 1000 + .5) / 1000
            TotalPrecip = TotalPrecip + r.Rain(i)
            RainLastStep = RainThisStep
Next i

ActualPrecip = 0
PeakPrecip = 0
For i = 1 To r.NumberSteps
          If r.Rain(i) < 0 Then r.Rain(i) = 0
          r.Rain(i) = r.Rain(i) * r.Total / TotalPrecip
          r.Rain(i) = Int(r.Rain(i) * 1000 + .5) / 1000
          ActualPrecip = ActualPrecip + r.Rain(i)
          If r.Rain(i) > PeakPrecip Then
                PeakPrecip = r.Rain(i)
                peakstep = i
          End If
Next i

r.Rain(peakstep) = r.Rain(peakstep) + (r.Total - ActualPrecip)
r.Rain(peakstep) = Int(r.Rain(peakstep) * 1000 + .5) / 1000
End Sub

Sub NewHydrograph (TH As HydrographType)
Dim i       As Integer

TH.TotalExcess = 0
TH.TotalRunoff = 0
TH.TotalInfiltration = 0
TH.TimeCon = 0
TH.TimeStep = 0
TH.HydroSteps = 0
TH.TotalArea = 0

For i = 1 To MAXSTEPS
      TH.outflow(i) = 0
      TH.Runoff(i) = 0
      TH.infiltration(i) = 0
Next i
End Sub

Sub NewRainfall (TR As rainfalltype)
Dim i   As Integer
TR.FileName = "RAINFALL.RNF"
TR.Total = 0
TR.NumberSteps = 0
TR.Duration = 0
TR.TimeStep = 0
For i = 1 To MAXSTEPS
        TR.Rain(i) = 0
Next i
End Sub

Sub NewShedFile (TW As WatershedType)
TW.InfMeth = 0

TW.FileName = "WATRSHED.SHD"
TW.TotalArea = 0
TW.TimeCon = 0
TW.ImpArea = 0
TW.DCIA = 0
w.AddAbsImp = 0
TW.AddAbsPerv = 0
TW.MaxInf = 0
TW.HIC = 0
TW.HIO = 0
TW.HK = 0

TW.CN = 0
TW.InitAbs = 0

End Sub

Sub OpenFile (FileType As Integer)
Dim Cancel           As Integer
Dim Success          As Integer

Select Case FileType
        Case 1  'Watershed
              Ext$ = "*.shd"
              TT$ = "Watershed"
        Case 2  'Rainfall
              Ext$ = "*.rnf"
              TT$ = "Rainfall"
        Case 3  ' Hydrograph
              Ext$ = "*.hyd"
              TT$ = "Hydrograph"
        Case 4  ' Pond
              Ext$ = "*.pnd"
              TT$ = "Pond"

End Select

'FileOpen FileName$, PathName$, Ext$, "Open " + TT$ + " File", 0, 7, 0, Cancel
'If Cancel Then Exit Sub

Select Case FileType
      Case 1
            w.FileName = PathName$ + "\" + FileName$
            Open w.FileName For Input As #1
            Success = OpenShedFile%(w)
      Case 2
            r.FileName = PathName$ + "\" + FileName$
            Open r.FileName For Input As #1
            Success = OpenRainFile%(r)
      Case 3
            h.FileName = PathName$ + "\" + FileName$
            Open h.FileName For Input As #1
            Success = OpenShedFile%(w)
            Success = OpenRainFile%(r)
            Success = OpenHydrFile%(h)
      Case 4
            p.FileName = PathName$ + "\" + FileName$
            Open p.FileName For Input As #1
            Success = OpenPondFile%(p)
End Select
Close

If Success Then
        MsgBox (TT$ + " File Retrieved")
Else
        MsgBox (TT$ + " File NOT Retrieved")
End If

End Sub

Function OpenHydrFile% (TH As HydrographType)
Dim i       As Integer

Input #1, TH.hydrotype
Input #1, TH.KR
Input #1, TH.PeakAttFact
Input #1, TH.HydroSteps
Input #1, TH.TimeStep
Input #1, TH.TimeCon
Input #1, TH.TotalArea
For i = 1 To TH.HydroSteps
        Input #1, TH.infiltration(i), TH.Runoff(i), TH.outflow(i)
              TH.TotalInfiltration = TH.TotalInfiltration + TH.infiltration(i)
              TH.TotalRunoff = TH.TotalRunoff + TH.Runoff(i)
              TH.TotalOutflow = TH.TotalOutflow + TH.outflow(i)
              If TH.outflow(i) > TH.PeakOutflow Then TH.PeakOutflow = TH.outflow(i)
Next i
If TH.TotalArea <> 0 Then TH.TotalExcess = TH.TotalRunoff * 60 * TH.TimeStep / (TH.TotalArea * 3630)
OpenHydrFile% = True
'th.RatCoeff = th.TotalExcess / r.TotalRain
'th.RatCoeff = FIX(th.RatCoeff * 1000) / 1000
'r.TotalRain = FIX(r.TotalRain * 100) / 100
End Function

Function OpenOutput% ()

      On Local Error GoTo OpenOutputError
      Open ODest For Output As #1
      OpenOutput = True
Exit Function
OpenOutputError:
      MsgBox (Error$)
      Close
      OpenOutput% = False
      Exit Function

End Function

Function OpenPondFile% (TP As PondType)
Dim i       As Integer
Dim Success As Integer

Input #1, TP.peakstep
Input #1, TP.PeakInflow
Input #1, TP.MaxStage
Input #1, TP.minstage

Input #1, TP.MaxVolume
Input #1, TP.OrigVolume
Input #1, TP.MinStorage
Input #1, TP.TimeStep

Input #1, TP.NumberSteps
Input #1, TP.InitStage
For i = 1 To TP.NumberSteps
                Input #1, TP.pstage(i)
                Input #1, TP.inflow(i)
                Input #1, TP.outflow(i)
Next i

Input #1, TP.NumberSSD
For i = 1 To TP.NumberSSD
                Input #1, TP.Stage(i)
                Input #1, TP.Discharge(i)
                Input #1, TP.Storage(i)
                Input #1, TP.srfarea(i)
Next i

Input #1, TP.cw
Input #1, TP.NumberWeirs
For i = 1 To TP.NumberWeirs
                Success = OpenWeirFile%(TP.Weir(i))
Next i
OpenPondFile% = Success
End Function

Function OpenRainFile% (TR As rainfalltype)
Dim i           As Integer

If TR.FileName = "" Then
      OpenRainFile% = False
End If
TR.Total = 0

Input #1, TR.NumberSteps, TR.Duration, TR.TimeStep
For i = 1 To TR.NumberSteps
              Input #1, TR.Rain(i)
              TR.Total = TR.Total + TR.Rain(i)
Next i

OpenRainFile% = True
End Function

Function OpenShedFile% (TW As WatershedType)

If TW.FileName = "" Then
      OpenShedFile% = False
End If

Input #1, TW.TotalArea
Input #1, TW.TimeCon
Input #1, TW.ImpArea
Input #1, TW.DCIA
Input #1, TW.AddAbsImp
Input #1, TW.AddAbsPerv
Input #1, TW.MaxInf
Input #1, TW.InfMeth

Select Case TW.InfMeth
        Case HORTON_INFILTRATION
                Input #1, TW.HIC
                Input #1, TW.HIO
                Input #1, TW.HK
        Case SCS_INFILTRATION
                Input #1, TW.CN
                Input #1, TW.InitAbs
End Select

If TW.DCIA <> 0 Then TW.FracDCIA = (TW.ImpArea / TW.TotalArea) * (TW.DCIA / 100)
If TW.TotalArea <> 0 Then
                TW.FracDCIA = (TW.DCIA / 100) * TW.ImpArea / TW.TotalArea
                TW.FracNDCIA = (TW.ImpArea / TW.TotalArea) * ((100 - TW.DCIA) / 100)
                TW.FracPerv = (TW.TotalArea - TW.ImpArea) / TW.TotalArea
End If

OpenShedFile% = True

End Function

Function OpenWeirFile% (TW As WeirType)
Dim i       As Integer

Input #1, TW.Label
Input #1, TW.Equation
Input #1, TW.Type

Input #1, TW.Invert
Input #1, TW.Diameter
Input #1, TW.Width
Input #1, TW.C
Input #1, TW.n

Input #1, TW.NumberSSD
For i = 1 To TW.NumberSSD
                Input #1, TW.Stage(i)
                Input #1, TW.Discharge(i)
Next i
OpenWeirFile% = True
End Function

Sub PlotAxis (xmax As Single, ymax As Single)
Dim i       As Integer
Dim Yint    As Integer
ReDim xlabel(11) As Single
ReDim ylabel(10) As Single

Yint = 12

  ' Draw Axes

'Line (90, 280)-(90, 0)
'Line (90, 280)-(640, 280)

  ' ***********  Draw Tick Marks on Y axis *******************

xlabel(1) = 0
For i = 0 To 10
 '               Line (85, 280 - i * Yint)-(95, 280 - i * Yint)
Next i

  ' ********** Draw Tick Marks on X Axis **********************

For i = 1 To 10
  '              Line (90 + i * 55, 275)-(90 + i * 55, 285)
                xlabel(i + 1) = (xmax / 600 + xlabel(i))
                ylabel(i) = (ymax * i / 10)
Next i

'LOCATE 3, 48: PRINT "Peak Flow:"; : PRINT USING "###.# cfs"; ymax
'LOCATE 4, 48: PRINT PlotLabel$

cfs$ = "Flow  cfs"
For i = 1 To 9
                'LOCATE i + 7, 1:  Print Mid$(cfs$, i, 1)
Next i
'LOCATE 23, 30
'Print "Time (hrs)"

kgk = 0
For i = 1 To 10
 '               LOCATE 22, 10 + kgk
                'PRINT USING "##.#"; xlabel(i)
                kgk = kgk + 7
Next i

'LOCATE 21, 4:  PRINT USING "###.#"; 0
'LOCATE 19, 4:  PRINT USING "###.#"; ylabel(2)
'LOCATE 17, 4:  PRINT USING "###.#"; ylabel(4)
'LOCATE 15, 4:  PRINT USING "###.#"; ylabel(6)
'LOCATE 13, 4:  PRINT USING "###.#"; ylabel(8)
'LOCATE 12, 4:  PRINT USING "###.#"; ylabel(10)

End Sub

Sub PlotHydrograph (TH As HydrographType, TR As rainfalltype)

If TH.HydroSteps = 0 Then
                MsgBox ("You must generate or retrieve a hydrograph before plotting")
                Exit Sub
End If

'Cls
'SCREEN 9, 0
'COLOR 1, 15

PlotAxis CSng(TH.HydroSteps * TR.TimeStep), TH.PeakOutflow
PlotSeries TH.HydroSteps, TH.HydroSteps, TH.outflow(), TH.PeakOutflow, CSng(TR.TimeStep)
PlotHyetograph CSng(TH.HydroSteps * TH.TimeStep), TH, TR

While INKEY$ = "":  Wend
'SCREEN 0, 0, 0

End Sub

Sub PlotHyetograph (xmax As Single, TH As HydrographType, TR As rainfalltype)
Dim i       As Integer
Dim x1      As Single
Dim x2      As Single
Dim y1      As Single
Dim y2      As Single
Dim Excess  As Single
x1 = 0
For i = 13 To 104 Step 13
'COLOR 1, 15:  Line (87, i)-(93, i)
'COLOR 7, 15:  Line (93, i)-(600, i)
Next i
'COLOR 1, 15


For i = 1 To TR.NumberSteps - 1

                y1 = (TR.Rain(i) * 50)
                If y1 < 1 Then y1 = 1
                y2 = 50 * ((TH.TimeStep * 60 * TH.Runoff(i) * 12) / (TH.TotalArea * 43560))
                If y2 < 1 Then y2 = 1
                x1 = 90 + ((i - 1) * TR.TimeStep) / xmax * 550
                x2 = 90 + (i * TR.TimeStep) / xmax * 550

'                COLOR 6, 15        ' *****  Make Box around Excess ***
'                Line (x1, 1)-(x2, 1)    ' Axis
'                Line (x1, y2)-(x2, y2)  ' Top
'                Line (x1, 1)-(x1, y2)   ' Left Side
'                Line (x2, y2)-(x2, 1)   ' Right Side
  '        IF x1 + 1 < x2 AND y2 > 2 THEN PAINT (x1 + 1, 2), CHR$(1), 6

'                COLOR 1, 15        '*****  Make Box around Rain *****

'                If y1 > y2 Then Line (x1, y2 + 1)-(x2, y2 + 1) ' Bottom
'                Line (x1, y1)-(x2, y1)  'Top
'                Line (x1, y2)-(x1, y1)  'Left
'                Line (x2, y1)-(x2, y2)  'Right
  '        IF (y1 > (y2 + 2)) AND x2 > x1 + 1 THEN PAINT (x1 + 1, y2 + 2), CHR$(128), 1
Next i
'COLOR 1, 15
'Line (90, 1)-(x2, 1)

'LOCATE 4, 6:  Print "1 in"
'LOCATE 8, 6:  Print "2 in"
'LOCATE 1, 1:  COLOR 6, 15:  Print "Excess"
'LOCATE 2, 1:  COLOR 1, 15:  Print "Rainfall"


End Sub

Sub PlotPondRouting (TP As PondType)
If TP.NumberSteps = 0 Then
                MsgBox ("You must route before plotting")
                Exit Sub
End If
'Cls
'SCREEN 9, 0
'COLOR 1, 7
PlotAxis CSng(TP.NumberSteps * TP.TimeStep), TP.PeakOutflow
PlotSeries TP.NumberSteps, TP.NumberSteps, TP.inflow(), TP.PeakInflow, TP.TimeStep
'LOCATE 5, 15:  Print "Flow into Pond":  Line (270, 62)-(310, 62)
'LOCATE 5, 50:  Print "Peak: " + Str$(TP.PeakInflow)
'COLOR 6, 7
PlotSeries TP.NumberSteps, TP.NumberSteps, TP.outflow(), TP.PeakInflow, TP.TimeStep
'LOCATE 6, 15:  Print "Flow out of Pond":  Line (270, 78)-(310, 78)
'LOCATE 6, 50:  Print "Peak: " + Str$(TP.PeakOutflow)


While INKEY$ = "":  Wend
'SCREEN 0, 0, 0

End Sub

Sub PlotSeries (Steps As Integer, StepsMax As Integer, Valy() As Single, ymax As Single, TimeStep As Single)
Dim j       As Integer
Dim i       As Integer
Dim xmax    As Single
Dim x1      As Single
Dim x2      As Single
Dim y1      As Single
Dim y2      As Single

ReDim valx(1 To Steps + 1)  As Single

xmax = CSng(StepsMax * TimeStep)
For j = 1 To Steps
                valx(j) = CSng(j * TimeStep)
                If valx(j) > xmax Then xmax = valx(j)
                If Valy(j) > ymax Then ymax = Valy(j)
Next j

  ' Draw Hydrograph Plot

For i = 1 To Steps
      x1 = 90 + valx(i) / xmax * 550
      y1 = 280 - Valy(i) / ymax * 120
      x2 = 90 + valx(i + 1) / xmax * 550
      y2 = 280 - Valy(i + 1) / ymax * 120

'      If x2 >= x1 Then Line (x1, y1)-(x2, y2)
Next i
End Sub

Sub PrintHydrograph (TH As HydrographType, TR As rainfalltype)
Dim CumInf      As Single
Dim CumRain     As Single
If TH.TotalArea = 0 Then
   Exit Sub
End If
page = 0
CumRain! = 0
i = 1
GoSub HydrographHeader
TH.TotalRunoff = 0
TH.TotalOutflow = 0
While i < TH.HydroSteps

                CumRain = CumRain + TR.Rain(i)
                CumInf = CumInf + TH.infiltration(i)
                TH.TotalRunoff = TH.TotalRunoff + TH.Runoff(i) * TH.TimeStep * 60
                TH.TotalOutflow = TH.TotalOutflow + TH.outflow(i) * TH.TimeStep * 60

                'PRINT #1, USING "       ##.##"; i * TR.TimeStep / 60;
                'PRINT #1, USING "       ##.##"; TR.Rain(i);
                'PRINT #1, USING "      ###.##"; CumRain;
                'PRINT #1, USING "       ##.##"; TH.infiltration(i);
                'PRINT #1, USING "     ####.##"; TH.Runoff(i);
                'PRINT #1, USING "      ###.##"; TH.outflow(i)
                i = i + 1
                If i / 17 = Int(i / 17) And OType = OSCREEN Then
                        While INKEY$ = "":  Wend
                        GoSub HydrographHeader
                End If

                If i = TH.HydroSteps Then
                                Print #1, String$(76, "-")
                                'PRINT #1, USING "       Totals over ####.## ac  ##.## in"; TH.TotalArea; CumRain;
                                'PRINT #1, USING "    ##.## in"; CumInf;
                                'PRINT #1, USING "    ##.## in    ##.## in"; 12 * TH.TotalRunoff / (TH.TotalArea * 43560); 12 * TH.TotalOutflow / (TH.TotalArea * 43560)
                                'PRINT #1, USING "       Totals (cf*1000)        #####   "; (CumRain / 12000) * (TH.TotalArea * 43560);
                                'PRINT #1, USING " ########   "; (CumInf / 12000) * (TH.TotalArea * 43560);
                                'PRINT #1, USING " ########    ########   "; TH.TotalRunoff / 1000; TH.TotalOutflow / 1000

                                If CumRain <> 0 Then TH.RatCoeff = (CumRain - CumInf) / CumRain
                                Print #1, Tab(9);
                                'PRINT #1, USING "Rational Coefficient    = #.###"; TH.RatCoeff;
                                'Print #1, Tab(42);
                                'PRINT #1, USING "Peak Flow = ####.## cfs"; TH.PeakOutflow;
                                'Print #1, Tab(9);
                                'If Left$(TH.hydrotype, 5) = "Santa" Then
                                 '               PRINT #1, USING "Routing Coefficient    = #.###"; TH.KR
                                Else
                                 '               PRINT #1, USING "Peak Attenuation Factor = ###"; TH.PeakAttFact
                                End If
                'End If
Wend
If OType = OSCREEN Then While INKEY$ = "":  Wend
Exit Sub
HydrographHeader:
                'If OType = OSCREEN Then Cls
                page = page + 1
'                Print #1, Tab(10); String$(48, "-"); " SMADA PAGE"; page
'                Print #1, Tab(8); Label$; "    Hydrograph Type :"; TH.hydrotype
'                Print #1, String$(76, "-")
'                Print #1, "       Time      Rain   Cumulative  Infiltration Instantaneous  Outflow "
'                Print #1, "       (hr)      (in)     (in)          (in)         (cfs)       (cfs)"
'                Print #1, String$(76, "-")
Return
End Sub

Sub PrintPondDesign (TP As PondType)
Dim i       As Integer

GoSub PrintPondDesignHeader
For i = 1 To TP.NumberSSD
      'PRINT #1, USING "        ###.##   ###.##    ####.##      ####.##    #####.##"; TP.Stage(i); TP.Storage(i); TP.Weir(1).Discharge(i); TP.Weir(2).Discharge(i); TP.Discharge(i)
Next i
While INKEY$ = "":  Wend
Print #1, :  Print #1,
For i = 1 To TP.NumberWeirs
      'PRINT #1, USING "        Weir ## Type       : "; i; :  PRINT #1, TP.Weir(1).Label
      'PRINT #1, USING "        Weir ## Equation   : "; i; :  PRINT #1, TP.Weir(1).Equation
Next i
While INKEY$ = "":  Wend

Exit Sub
PrintPondDesignHeader:
'                Cls
'                Print #1, String$(76, "-")
       '         Print #1, "       Stage    Storage     Weir 1 Q     Weir 2 Q    Discharge"
       '         Print #1, "       (ft)     (ac-ft)       (cfs)        (cfs)       (cfs)"
       '         Print #1, String$(76, "-")
Return


Return
End Sub

Sub PrintPondRouting (TP As PondType)
Dim i       As Integer
Dim page    As Integer

i = 1
GoSub PrintPondRoutingHeader
While i < TP.NumberSteps
                'PRINT #1, USING "        ###.##"; TP.TimeStep * i / 60;
                'PRINT #1, USING "   ####.##"; TP.inflow(i);
                'PRINT #1, USING "   ####.##"; TP.outflow(i);
                'PRINT #1, USING "      ####.##"; TP.pstage(i)

                i = i + 1
                If i / 19 = Int(i / 19) Then
                        While INKEY$ = "":  Wend
                        GoSub PrintPondRoutingHeader
                End If
Wend
While INKEY$ = "":  Wend
Exit Sub
PrintPondRoutingHeader:
'                Cls
                page = page + 1
'                Print #1, String$(56, "-"); " SMADA PAGE"; page
'                Print #1, "         Time    Inflow     Outflow      Stage "
'                Print #1, "         (hr)     (cfs)      (cfs)        (ft)"
'                Print #1, String$(72, "-")
Return
End Sub

Sub PrintWatershed (TW As WatershedType)
'If OType = OSCREEN Then Cls
'Print #1, String$(15, "-"); "  SMADA 5.0 Unregistered DOS Version "; String$(24, "-")
'Print #1, String$(20, "-"); "  Watershed Information   "; String$(30, "-")
'Print #1, String$(76, "-")

'PRINT #1, USING "        Watershed Total Area (acres)     :#####.##"; TW.TotalArea
'PRINT #1, USING "        Impervious Area (acres)          :#####.##"; TW.ImpArea
'PRINT #1, USING "        Time of Concentration (min)      :  ###.#"; TW.TimeCon
'PRINT #1, USING "        % Impervious Directly Connected  :   ##.##"; TW.DCIA
'Print #1, String$(76, "-")
'Print #1, Tab(15); "Additional Abstraction"
'PRINT #1, USING "        Over Pervious Area (inches)      :   ##.##"; TW.addabsPerv
'PRINT #1, USING "        Over Impervious Area (inches)    :   ##.##"; TW.AddAbsImp
'Print #1, String$(76, "-")
'Print #1, Tab(15); "Infiltration Characteristics: "; TW.Label
'PRINT #1, USING "        Max Infiltration Capacity (in)   :  ###.##"; TW.MaxInf

If TW.InfMeth = SCS_INFILTRATION Then
'      PRINT #1, USING "        SCS Curve Number for Pervious    :  ###"; TW.CN
'      PRINT #1, USING "        Initial Abstraction Factor       :    #.##"; TW.InitAbs
ElseIf TW.InfMeth = HORTON_INFILTRATION Then
 '     PRINT #1, USING "        Horton Initial Rate (in/hr)      :  ###.##"; TW.HIO
  '    PRINT #1, USING "        Horton Limiting Rate (in/hr)     :  ###.##"; TW.HIC
  '    PRINT #1, USING "        Depletion Coefficient (/hr)      :   ##.###"; TW.HK
End If

If OType = OSCREEN Then While INKEY$ = "":  Wend
'If OType = OPRINTER Then Print #1, Chr$(12)
End Sub

Sub RoundRainfall (places As Integer, TR As rainfalltype)

TR.Total = 0
For i = 1 To TR.NumberSteps
      TR.Total = TR.Total + TR.Rain(i)
      TR.Total = Int(TR.Total * 10 ^ places + .5) / 10 ^ places
Next i

End Sub

Sub RouteHYDRtoPOND (TH As HydrographType, TP As PondType)

Dim i           As Integer
Dim j           As Integer
Dim k           As Integer
Dim ExitFlag    As Integer

Dim Volume      As Single
Dim tnexts      As Single
Dim flow        As Single

TP.PeakInflow = 0
For i = 1 To TH.HydroSteps
      TP.inflow(i) = TH.outflow(i)
      If TP.PeakInflow < TP.inflow(i) Then TP.PeakInflow = TP.inflow(i)
Next i

TP.TimeStep = TH.TimeStep
If TP.InitStage = 0 Then TP.InitStage = TP.Stage(1)
Volume = getVal!(TP.NumberSSD, TP.InitStage, TP.Stage(), TP.Storage()) * 43560
If Volume < 0 Then Volume = 0

i = 1
TP.MaxVolume = 0
TP.MaxStage = 0
TP.NumberSteps = TH.HydroSteps
TP.PeakOutflow = 0
While Not ExitFlag
      For k = 1 To 10
            TP.pstage(i) = getVal!(TP.NumberSSD, Volume / 43560!, TP.Storage(), TP.Stage())
            flow = getVal!(TP.NumberSSD, Volume / 43560, TP.Storage(), TP.Discharge())
            If flow < 0 Then flow = 0
            TP.outflow(i) = flow
            Volume = Volume + (TP.inflow(i) - TP.outflow(i)) * TH.TimeStep * 6
            If Volume > TP.MaxVolume Then TP.MaxVolume = Volume
            If flow > TP.PeakOutflow Then TP.PeakOutflow = flow
            If TP.pstage(i) > TP.MaxStage Then TP.MaxStage = TP.pstage(i)
      Next k
      i = i + 1
      If i = MAXSTEPS Then ExitFlag = True
      If i > TH.HydroSteps And TP.outflow(i) < .01 * TP.PeakOutflow Then ExitFlag = True
Wend
TP.NumberSteps = i - 1
End Sub

Sub SantaBarbara (TH As HydrographType)
Dim i           As Integer
Dim ExitFlag    As Integer

h.PeakOutflow = 0
h.TotalOutflow = 0
h.Runoff(1) = 0
h.outflow(1) = 0
ExitFlag = False
h.hydrotype = "Santa Barbara Method"

If TH.TimeCon = 0 Then Exit Sub
If TH.TimeStep = 0 Then Exit Sub

i = 1
h.KR = TH.TimeStep / (2 * TH.TimeCon + TH.TimeStep)

While Not ExitFlag
                If i >= 2 Then
                                TH.outflow(i) = TH.outflow(i - 1) + TH.KR * (TH.Runoff(i - 1) + TH.Runoff(i) - 2 * TH.outflow(i - 1))
                Else
                                TH.outflow(i) = TH.KR * TH.Runoff(i)
                End If
                If TH.outflow(i) > TH.PeakOutflow Then TH.PeakOutflow = TH.outflow(i)

                If i >= 5 Then
                                If ((TH.outflow(i) < .005 * TH.PeakOutflow) And (i > TH.HydroSteps)) Or (i = MAXSTEPS - 1) Then
                                                h.HydroSteps = i
                                                ExitFlag = True
                                End If
                End If
                i = i + 1
Wend
End Sub

Sub SaveFile (FileType As Integer)
Dim Cancel       As Integer
Dim Success         As Integer

Select Case FileType
      Case 1
            TT$ = "Watershed"
            DeFile$ = "WATRSHED.SHD"
      Case 2
            TT$ = "Rainfall"
            DeFile$ = "RAINFALL.RNF"
      Case 3
            TT$ = "Hydrograph"
            DeFile$ = "HYDRGRPH.HYD"
      Case 4
            TT$ = "Pond"
            DeFile$ = "POND.PND"
End Select

'FileSave FileName$, PathName$, DeFile$, "Save " + TT$ + " File As", 0, 7, 0, Cancel
'If Cancel Then Exit Sub

Select Case FileType
      Case 1
            w.FileName = PathName$ + "\" + FileName$
            Open w.FileName For Output As #1
            Success = SaveShedFile(w)
      Case 2
            r.FileName = PathName$ + "\" + FileName$
            Open r.FileName For Output As #1
            Success = SaveRainFile(r)
      Case 3
            h.FileName = PathName$ + "\" + FileName$
            Open h.FileName For Output As #1
            Success = SaveShedFile%(w)
            Success = SaveRainFile%(r)
            Success = SaveHydrFile%(h)
      Case 4
            p.FileName = PathName$ + "\" + FileName$
            Open p.FileName For Output As #1
            Success = SavePondFile%(p)
End Select
Close

If Success Then
      MsgBox (TT$ + " File Saved")
Else
      MsgBox (TT$ + " File NOT Saved")
End If

End Sub

Function SaveHydrFile% (TH As HydrographType)
Dim i       As Integer
Print #1, TH.hydrotype
Print #1, TH.KR
Print #1, TH.PeakAttFact
Print #1, TH.HydroSteps
Print #1, TH.TimeStep
Print #1, TH.TimeCon
Print #1, TH.TotalArea
For i = 1 To TH.HydroSteps
        Print #1, TH.infiltration(i), TH.Runoff(i), TH.outflow(i)
Next i
SaveHydrFile% = True
End Function

Function SavePondFile% (TP As PondType)
Dim i       As Integer
Dim Success As Integer

Print #1, TP.peakstep
Print #1, TP.PeakInflow
Print #1, TP.MaxStage
Print #1, TP.minstage

Print #1, TP.MaxVolume
Print #1, TP.OrigVolume
Print #1, TP.MinStorage
Print #1, TP.TimeStep

Print #1, TP.NumberSteps
Print #1, TP.InitStage
For i = 1 To TP.NumberSteps
                Print #1, TP.pstage(i)
                Print #1, TP.inflow(i)
                Print #1, TP.outflow(i)
Next i

Print #1, TP.NumberSSD
For i = 1 To TP.NumberSSD
                Print #1, TP.Stage(i)
                Print #1, TP.Discharge(i)
                Print #1, TP.Storage(i)
                Print #1, TP.srfarea(i)
Next i

Print #1, TP.cw
Print #1, TP.NumberWeirs
For i = 1 To TP.NumberWeirs
                Success = SaveWeirFile%(TP.Weir(i))
Next i
SavePondFile% = Success
End Function

Function SaveRainFile% (TR As rainfalltype)
Dim i       As Integer
If TR.TimeStep = 0 Then
                SaveRainFile% = False
                Exit Function
End If

Print #1, TR.NumberSteps, TR.Duration, TR.TimeStep
For i = 1 To TR.NumberSteps
          Print #1, TR.Rain(i)
Next i

SaveRainFile% = True
End Function

Function SaveShedFile% (TW As WatershedType)

If TW.InfMeth = 0 Then
                SaveShedFile% = False
                Exit Function
End If

Print #1, TW.TotalArea
Print #1, TW.TimeCon
Print #1, TW.ImpArea
Print #1, TW.DCIA
Print #1, TW.AddAbsImp
Print #1, TW.AddAbsPerv
Print #1, TW.MaxInf
Print #1, TW.InfMeth

Select Case TW.InfMeth
        Case HORTON_INFILTRATION
                Print #1, TW.HIC
                Print #1, TW.HIO
                Print #1, TW.HK
        Case SCS_INFILTRATION
                Print #1, TW.CN
                Print #1, TW.InitAbs
        End Select

SaveShedFile% = True
End Function

Function SaveWeirFile% (TW As WeirType)
Dim i   As Integer
Print #1, TW.Label
Print #1, TW.Equation
Print #1, TW.Type

Print #1, TW.Invert
Print #1, TW.Diameter
Print #1, TW.Width
Print #1, TW.C
Print #1, TW.n

Print #1, TW.NumberSSD
For i = 1 To TW.NumberSSD
                Print #1, TW.Stage(i)
                Print #1, TW.Discharge(i)
Next i
SaveWeirFile% = True
End Function

Sub ScaleRainfall (r As rainfalltype)
Dim TotalPrecip         As Single

RoundRainfall 3, r

If r.Total = 0 Then Exit Sub
'a$ = "You may scale an existing rainfall file to represent another|"
'a$ = a$ + "total rainfall.|"
'a$ = a$ + "The existing total rainfall is " + STR$(r.TotalRain)
'a$ = a$ + " (in).| The entire rainfall will be scaled to represent the|"
'a$ = a$ + "new total while the shape will be preserved."
'dummy = Alert(4, a$, 10, 10, 18, 70, "Cancel", "Continue", "")
'IF dummy = 0 OR dummy = 1 THEN EXIT SUB

'TotalPrecip = VAL(GetLabel$("", "Input New Total Precipitation (in):"))
If TotalPrecip = 0 Then Exit Sub

For i = 1 To r.NumberSteps
                r.Rain(i) = r.Rain(i) * TotalPrecip / r.Total
                r.Rain(i) = Int(r.Rain(i) * 1000 + .5) / 1000
Next i
r.Total = 0
For i = 1 To r.NumberSteps
      r.Total = r.Total + r.Rain(i)
Next i
If r.Total <> TotalPrecip Then
      r.Rain(r.NumberSteps) = r.Rain(r.NumberSteps) + (TotalPrecip - r.Total)
      r.Rain(r.NumberSteps) = Int(r.Rain(r.NumberSteps) * 1000 + .5) / 1000
End If

End Sub

Sub SCSMethod (PAF As Single, TH As HydrographType)

ReDim UnitHydro(MAXSTEPS)    As Single

Dim i               As Integer
Dim j               As Integer
Dim Jsteps          As Integer
Dim ExitFlag        As Integer
Dim StepTime        As Single
Dim UnitTimeCon     As Single
Dim TimeFall        As Single
Dim FallLimbFact    As Single
Dim TotalUnitTime   As Single
Dim Peak            As Single
Dim TotalHydro      As Single
'DIM p1              AS SINGLE

If TH.TimeCon = 0 Then Exit Sub
If TH.TimeStep = 0 Then Exit Sub

For j = 1 To TH.HydroSteps
                TH.outflow(j) = 0
Next j

TH.TotalOutflow = 0
TH.PeakOutflow = 0
While PAF <= 50 Or PAF > 645
                a1$ = "Choose a value for the Peak Attenuation Factor" + Chr$(13)
                a1$ = a1$ + Chr$(13)
                a1$ = a1$ + "Value should be between 50 and 645"
                PAF = Val(InputBox$(a1$, "Input Peak Attenuation Factor", "484"))
                If PAF = 0 Then Exit Sub
Wend
TH.PeakAttFact = PAF
TH.hydrotype = "SCS " + Str$(PAF) + " Hydrograph"

  ' *****************************************************
'
  ' ===========  Generate Unit Hydrograph ===============
'
  ' *****************************************************

FallLimbFact = (1291 / TH.PeakAttFact) - 1
If TH.TimeCon < TH.TimeStep Then
                UnitTimeCon = TH.TimeStep
Else
                UnitTimeCon = TH.TimeCon
End If

TimeFall = FallLimbFact * TH.TimeCon
TotalUnitTime = UnitTimeCon + TimeFall
TotalUnitTime = Int(TotalUnitTime / TH.TimeStep + .5) * TH.TimeStep
Jsteps = Int(TotalUnitTime / TH.TimeStep + .5)
For j = 1 To Jsteps
      StepTime = j * r.TimeStep
      If StepTime <= UnitTimeCon Then
                  UnitHydro(j) = StepTime / UnitTimeCon
      Else
                    UnitHydro(j) = 1 + UnitTimeCon / TimeFall - StepTime / TimeFall
      End If
Next j


'IF w.SmoothFlag = TRUE THEN
  '        FOR j = 1 TO Jsteps
  '                Steptime = j * r.RainTimeStep
  '                tr! = Steptime / UnitTimeCon
  '                IF Steptime <= UnitTimeCon THEN
  '                        UnitHydro(j) = .05903 - .7650173 * tr! + 4.415467 * tr! ^ 2 - 2.708 * tr! ^ 3
  '                        tp = j
  '                ELSE
  '                        IF th.PeakAttFact = 484 THEN a! = 2.445061: b! = -1.763459: c! = .425012: c! = .425012: D! = -.034003
  '                        IF th.PeakAttFact = 256 THEN a! = 1.883346: b! = -.9724275: c! = .1670826: D! = -.009394733#
  '                        UnitHydro(j) = a! + b! * tr! + c! * tr! ^ 2 + D! * tr! ^ 3
  '                END IF
  '        IF UnitHydro(j) < 0 THEN UnitHydro(j) = 0
  '        NEXT j
'END IF

TotalHydro = 0
For j = 1 To Jsteps
                TotalHydro = TotalHydro + UnitHydro(j)
Next j


For j = 1 To Jsteps
                UnitHydro(j) = UnitHydro(j) / TotalHydro
                'IF UnitHydro(j) > p1 THEN p1 = UnitHydro(j)
Next j

  ' ==========   Generate Hydrograph using unit Hydrograph ===============

i = 1
ExitFlag = False
While Not ExitFlag

                For j = 0 To i - 1
                                TH.outflow(i) = TH.outflow(i) + TH.Runoff(i - j) * UnitHydro(j + 1)
                Next j

                If TH.outflow(i) > TH.PeakOutflow Then TH.PeakOutflow = TH.outflow(i)

                If i = MAXSTEPS Then ExitFlag = True
                If i > TH.HydroSteps And TH.outflow(i) < .005 * TH.PeakOutflow Then ExitFlag = True

                i = i + 1
Wend
TH.HydroSteps = i - 1

End Sub

