Const MAXNODES = 8
Const MAXUNIT = 100
Const MAXSTEPS = 260
Const SSD_RELATIONSHIPS = 20

Const OSCREEN = 0
Const OPRINTER = 1
Const OFILE = 2

Const VNOTCH60 = 0
Const VNOTCH90 = 1
Const SCRECT = 2
Const BCRECT = 3
Const ORIFICE = 4

'Infiltration Types

Const SCS_INFILTRATION = 100
Const HORTON_INFILTRATION = 101

' Types of Rainfall

Const USER_RAINFALL = 100
Const SCSTYPEII_RAINFALL = 101
Const SCSTYPEIIFL_RAINFALL = 102
Const SCSTYPEIII_RAINFALL = 103
Const CONSTANT_RAINFALL = 104

'$INCLUDE: 'SM_type.bi'
'$INCLUDE: 'SM_calc.bi'

Sub CalculatePondDischarge (Pond As PondType)
Dim i       As Integer
Dim j       As Integer

For i = 1 To Pond.NumberSSD
   Pond.Discharge(i) = 0
Next i

For i = 1 To Pond.NumberWeirs
   Pond.Weir(i).NumberSSD = Pond.NumberSSD
   For j = 1 To Pond.NumberSSD
      Pond.Weir(i).Stage(j) = Pond.Stage(j)
   Next j
      CalculateWeirDischarge Pond.Weir(i)
Next i

For i = 1 To Pond.NumberWeirs
   For j = 1 To Pond.NumberSSD
      Pond.Discharge(j) = Pond.Discharge(j) + Pond.Weir(i).Discharge(j)
   Next j
Next i


End Sub

Sub CalculateWeirDischarge (W As WeirType)
Dim height      As Single

For j = 1 To W.NumberSSD
   height = W.Stage(j) - W.Invert
   If height < 0 Then height = 0

   Select Case W.Type
        Case VNOTCH60 To VNOTCH90   ' 60 Degree V-Notch
           W.Discharge(j) = W.C * (height ^ W.n)
        Case SCRECT To BCRECT       'Rectangular Weir
           W.Discharge(j) = W.C * W.Width * (height ^ W.n)
        Case 5  'Orifice
           If (height > W.Diameter) Then
                height = 2 * 32.2 * (height - W.Diameter / 2)
                W.Discharge(j) = W.C * 3.14 * (W.Diameter ^ 2 / 4) * (2 * 32.32 * height) ^ .5
           Else
                W.Discharge(j) = 2.5 * W.Diameter * (height ^ 2.5)
           End If
   End Select
Next j
End Sub

Sub GenerateInstantHydrograph (TW As WatershedType, TR As RainfallType, TH As HydrographType)
 
Dim i                                   As Integer
Dim j                                   As Integer
Dim PotentialInfiltration               As Single
Dim TotalPerviousRainfall               As Single
Dim CumulativeInfiltration              As Single
Dim CumulativePerviousAbstraction       As Single
Dim CumulativeImperviousAbstraction     As Single
Dim CumulativePerviousExcess            As Single
Dim LastCumulativePerviousRunoff        As Single
Dim CumulativePerviousRunoff            As Single
Dim ImperviousExcess                    As Single
Dim PerviousExcess                      As Single
Dim ImpervioustoPerviousExcess          As Single
Dim DummyCumInf                         As Single
Dim PerviousRunoff                      As Single
Dim ImperviousRunoff                    As Single
Dim TempExcess                          As Single


NewHydrograph TH
GenerateWatershed TW

CumulativePerviousExcess = 0
CumulativePerviousRunoff = 0
CumulativeInfiltration = 0
CumulativeImperviousAbstraction = 0
ImperviousExcess = 0
TR.total = 0
TotalPerviousRainfall = 0
PerviousExcess = 0
LastCumulativePerviousRunoff = 0


TH.TimeCon = TW.TimeCon
TH.timestep = TR.timestep
TH.hydrosteps = TR.numbersteps
TH.TotalArea = TW.TotalArea

For i = 1 To TR.numbersteps
   TR.total = TR.total + TR.rain(i)
   

'       **  Routing of Abstraction on Impervious Surface **

   If TW.AddAbsImp > TR.total Then
        ImperviousExcess = 0
        CumulativeImperviousAbstraction = CumulativeImperviousAbstraction + TR.rain(i)

   ElseIf TR.total - TR.rain(i) < TW.AddAbsImp And TR.total > TW.AddAbsImp Then
        ImperviousExcess = TR.rain(i) - (TW.AddAbsImp - CumulativeImperviousAbstraction)
        CumulativeImperviousAbstraction = TW.AddAbsImp
        If ImperviousExcess < 0 Then ImperviousExcess = 0

   Else
        ImperviousExcess = TR.rain(i)
        If ImperviousExcess < 0 Then ImperviousExcess = 0
   End If

   If TW.TotalArea <= TW.ImpArea Then
      ImpervioustoPerviousExcess = 0
   Else
      ImpervioustoPerviousExcess = TW.FracNDCIA * ImperviousExcess * TW.TotalArea / (TW.TotalArea - TW.ImpArea)
   End If
   ImperviousRunoff = ImperviousExcess
   PerviousRainfall = TR.rain(i) + ImpervioustoPerviousExcess
   TotalPerviousRainfall = PerviousRainfall + TotalPerviousRainfall

'       **      Routing of Abstraction on Pervious Surface      **

   If TW.addabsPerv > TotalPerviousRainfall Then
        PerviousExcess = 0
        CumulativePerviousAbstraction = TotalPerviousRainfall
   Else
        PerviousExcess = PerviousRainfall + (CumulativePerviousAbstraction - TW.addabsPerv)
        CumulativePerviousAbstraction = TW.addabsPerv
   End If

   CumulativePerviousExcess = CumulativePerviousExcess + PerviousExcess

   If TW.InfMeth = HORTON_INFILTRATION Then
        ExitFlag = False
        j = 0
        While Not ExitFlag
                DummyCumInf = (TW.HIC * j * TR.timestep) / 60! + (TW.HIO - TW.HIC) * (1 - Exp((TW.HK * j * TR.timestep / 60))) / TW.HK
                If CumulativeInfiltration > DummyCumInf Then
                        j = j + 1
                Else
                        DummyTime = j * TR.timestep / 60!
                        ExitFlag = True
                End If
        Wend
        PotentialInfiltration = TW.HIC + (TW.HIO - TW.HIC) * Exp((TW.HK * DummyTime)) * TR.timestep / 60

        If CumulativeInfiltration < TW.MaxInf Then
                If PerviousExcess > PotentialInfiltration Then
                        PerviousRunoff = PerviousExcess - PotentialInfiltration
                        TH.infiltration(i) = PotentialInfiltration
                Else
                        PerviousRunoff = 0
                        TH.infiltration(i) = PerviousExcess
                End If
        Else
                PerviousRunoff = PerviousExcess
                TH.infiltration(i) = 0
        End If
   End If

   If TW.InfMeth = SCS_INFILTRATION Then
        SPrime = 1000 / TW.CN - 10
        AbsFact = TW.InitAbs * SPrime
        If CumulativePerviousExcess > AbsFact Then
                CumulativePerviousRunoff = ((CumulativePerviousExcess - AbsFact) ^ 2) / (CumulativePerviousExcess + (1 - TW.InitAbs) * SPrime)
                PerviousRunoff = CumulativePerviousRunoff - LastCumulativePerviousRunoff
                LastCumulativePerviousRunoff = CumulativePerviousRunoff
                If CumulativeInfiltration < TW.MaxInf Then
                        TH.infiltration(i) = PerviousExcess - PerviousRunoff
                Else
                        TH.infiltration(i) = 0
                        PerviousRunoff = PerviousExcess
                End If
        Else
                If CumulativeInfiltration < TW.MaxInf Then
                        PerviousRunoff = 0
                        TH.infiltration(i) = PerviousExcess
                Else
                        PerviousRunoff = PerviousExcess
                        TH.infiltration(i) = 0
                End If
        End If
   End If

   CumulativeInfiltration = CumulativeInfiltration + TH.infiltration(i)
   TH.infiltration(i) = TH.infiltration(i) * TW.FracPerv
   TempExcess = TW.FracDCIA * ImperviousRunoff + TW.FracPerv * PerviousRunoff
   TH.TotalExcess = TempExcess + TH.TotalExcess
   TH.runoff(i) = 60 * TempExcess * TW.TotalArea * 1.008 / TR.timestep
   TH.TotalInfiltration = TH.TotalInfiltration + TH.infiltration(i)
Next i
End Sub

Sub GenerateSantaBarbara (TH As HydrographType)
Dim i           As Integer
Dim ExitFlag    As Integer

TH.PeakOutflow = 0
TH.TotalOutflow = 0
'TH.runoff(1) = 0
TH.outflow(1) = 0
ExitFlag = False
TH.hydrotype = "Santa Barbara Method"

If TH.TimeCon = 0 Then Exit Sub
If TH.timestep = 0 Then Exit Sub

i = 1
TH.KR = TH.timestep / (2 * TH.TimeCon + TH.timestep)

While Not ExitFlag
        If i >= 2 Then
                TH.outflow(i) = TH.outflow(i - 1) + TH.KR * (TH.runoff(i - 1) + TH.runoff(i) - 2 * TH.outflow(i - 1))
        Else
                TH.outflow(i) = TH.KR * TH.runoff(i)
        End If
        If TH.outflow(i) > TH.PeakOutflow Then TH.PeakOutflow = TH.outflow(i)

        If (i > TH.hydrosteps) And (TH.outflow(i) < .0005) Then
            TH.hydrosteps = i
            ExitFlag = True
        End If
        
        If ((TH.outflow(i) < .005 * TH.PeakOutflow) And (i > TH.hydrosteps)) Or (i = MAXSTEPS - 1) Then
            TH.hydrosteps = i
            ExitFlag = True
        End If
        i = i + 1
Wend
End Sub

Sub GenerateSCSMethod (PAF As Single, TH As HydrographType)

Dim TUH             As UnitHydrographType
Dim i               As Integer
Dim j               As Integer

Dim StepTime        As Single
Dim UnitTimeCon     As Single
Dim TimeFall        As Single
Dim FallLimbFact    As Single
Dim TotalUnitTime   As Single


If TH.TimeCon = 0 Then Exit Sub
If TH.timestep = 0 Then Exit Sub

For j = 1 To TH.hydrosteps
        TH.outflow(j) = 0
Next j

TH.TotalOutflow = 0
TH.PeakOutflow = 0
While PAF <= 50 Or PAF > 645
        a1$ = "Choose a value for the Peak Attenuation Factor" + Chr$(13)
        a1$ = a1$ + Chr$(13)
        a1$ = a1$ + "Value should be between 50 and 645"
        PAF = Val(InputBox$(a1$, "Input Peak Attenuation Factor", "484"))
        If PAF = 0 Then Exit Sub
Wend
TH.PeakAttFact = PAF
TH.hydrotype = "SCS " + Str$(PAF) + " Hydrograph"

' *****************************************************
'
' ===========  Generate Unit Hydrograph ===============
'
' *****************************************************

FallLimbFact = (1291 / TH.PeakAttFact) - 1
If TH.TimeCon < TH.timestep Then
        UnitTimeCon = TH.timestep
Else
        UnitTimeCon = TH.TimeCon
End If

TimeFall = FallLimbFact * TH.TimeCon
TotalUnitTime = UnitTimeCon + TimeFall
TotalUnitTime = Int(TotalUnitTime / TH.timestep + .5) * TH.timestep
TUH.numbersteps = Int(TotalUnitTime / TH.timestep + .5)
For j = 1 To TUH.numbersteps
   StepTime = j * TH.timestep
   If StepTime <= UnitTimeCon Then
         TUH.q(j) = StepTime / UnitTimeCon
   Else
          TUH.q(j) = 1 + UnitTimeCon / TimeFall - StepTime / TimeFall
   End If
Next j

GenerateUnitHydrograph TUH, TH

End Sub

Sub GenerateUnitHydrograph (TUH As UnitHydrographType, TH As HydrographType)
Dim i           As Integer
Dim j           As Integer
Dim ExitFlag    As Integer
Dim TotalHydro  As Single

TotalHydro = 0
For j = 1 To TUH.numbersteps
        TotalHydro = TotalHydro + TUH.q(j)
Next j


For j = 1 To TUH.numbersteps
        TUH.q(j) = TUH.q(j) / TotalHydro
Next j

' ==========   Generate Hydrograph using unit Hydrograph ===============

i = 1
ExitFlag = False
While Not ExitFlag

        For j = 0 To i - 1
                TH.outflow(i) = TH.outflow(i) + TH.runoff(i - j) * TUH.q(j + 1)
        Next j

        If TH.outflow(i) > TH.PeakOutflow Then TH.PeakOutflow = TH.outflow(i)

        If i = MAXSTEPS - 1 Then ExitFlag = True
        If i > TH.hydrosteps And TH.outflow(i) < .005 * TH.PeakOutflow Then ExitFlag = True
        If (i > TH.hydrosteps) And (TH.outflow(i) < .0005) Then ExitFlag = True
        i = i + 1
Wend
TH.hydrosteps = i - 1

End Sub

Sub GenerateWatershed (TW As WatershedType)
If TW.DCIA <> 0 Then TW.FracDCIA = (TW.ImpArea / TW.TotalArea) * (TW.DCIA / 100)
If TW.TotalArea <> 0 Then
        TW.FracDCIA = (TW.DCIA / 100) * TW.ImpArea / TW.TotalArea
        TW.FracNDCIA = (TW.ImpArea / TW.TotalArea) * ((100 - TW.DCIA) / 100)
        TW.FracPerv = (TW.TotalArea - TW.ImpArea) / TW.TotalArea
End If

End Sub

Function getVal! (n As Integer, xo As Single, x() As Single, y() As Single)

i = n
        While xo < x(i) And i > 0
                i = i - 1
        Wend
        
        If i < n Then
                If x(i + 1) <> x(i) Then
                        m! = (y(i + 1) - y(i)) / (x(i + 1) - x(i))
                Else
                        m! = 0
                End If

        Else
                If x(i - 1) <> x(i) Then
                        m! = (y(i) - y(i - 1)) / (x(i) - x(i - 1))
                Else
                        m! = 0
                End If

        End If
        b! = y(i) - m! * x(i)
        getVal! = m! * xo + b!

End Function

Sub MakeDimRainFall (r As RainfallType)

Dim i                   As Integer
Dim peakstep            As Integer
Dim a                   As Single
Dim PeakPrecip          As Single
Dim DimTime             As Single
Dim RainThisStep        As Single
Dim RainLastStep        As Single
Dim TotalPrecip         As Single
Dim Precip              As Single
Dim ActualPrecip        As Single


StartofMakeDimRainFall:
RainLastStep = 0
TotalPrecip = 0
For i = 1 To r.numbersteps
      DimTime = i / r.numbersteps
      Select Case r.Type
         Case SCSTYPEII_RAINFALL
                If DimTime <= 1 Then Precip = .0178 + 1.893 * DimTime - .9192 * DimTime ^ 2
                If DimTime = .5 Then Precip = .663
                If DimTime < .5 Then Precip = 22.394 - 101.8 * DimTime + 116.5 * DimTime ^ 2
                If DimTime < .44 Then Precip = .4395 * DimTime ^ 1.195
                If DimTime < .0271 Then Precip = .226 * DimTime
        
         Case SCSTYPEIIFL_RAINFALL
                If DimTime <= 1 Then Precip = -.0508 + 2.0277 * DimTime - .9855 * DimTime ^ 2
                If DimTime = .5 Then Precip = .607
                If DimTime < .5 Then Precip = .00953 + .0726 * DimTime + .9643 * DimTime ^ 2
                If DimTime < .0271 Then Precip = .226 * DimTime
        
         Case SCSTYPEIII_RAINFALL
                If DimTime <= 1 Then Precip = 1.003 * DimTime ^ .2657
                If DimTime < .7 Then Precip = -3.8077 + 13.7517 * DimTime - 10.009 * DimTime ^ 2
                If DimTime = .5 Then Precip = .5
                If DimTime <= .5 Then Precip = .01 * Exp(7.212 * DimTime)
                If DimTime < .0271 Then Precip = .224 * DimTime

        Case CONSTANT_RAINFALL
                Precip = i / r.numbersteps

      End Select
      RainThisStep = Precip * r.total
      r.rain(i) = (RainThisStep - RainLastStep)
      If r.rain(i) < .001 Then r.rain(i) = .001
      r.rain(i) = Int(r.rain(i) * 1000 + .5) / 1000
      TotalPrecip = TotalPrecip + r.rain(i)
      RainLastStep = RainThisStep
Next i

ActualPrecip = 0
PeakPrecip = 0
For i = 1 To r.numbersteps
     If r.rain(i) < 0 Then r.rain(i) = 0
     r.rain(i) = r.rain(i) * r.total / TotalPrecip
     r.rain(i) = Int(r.rain(i) * 1000 + .5) / 1000
     ActualPrecip = ActualPrecip + r.rain(i)
     If r.rain(i) > PeakPrecip Then
        PeakPrecip = r.rain(i)
        peakstep = i
     End If
Next i

r.rain(peakstep) = r.rain(peakstep) + (r.total - ActualPrecip)
r.rain(peakstep) = Int(r.rain(peakstep) * 1000 + .5) / 1000
End Sub

Sub NewHydrograph (TH As HydrographType)
Dim i       As Integer

TH.TotalExcess = 0
TH.TotalRunoff = 0
TH.TotalInfiltration = 0
TH.TimeCon = 0
TH.timestep = 0
TH.hydrosteps = 0
TH.TotalArea = 0

For i = 1 To MAXSTEPS
   TH.outflow(i) = 0
   TH.runoff(i) = 0
   TH.infiltration(i) = 0
Next i
End Sub

Sub NewPondFile (TP As PondType)
TP.peakstep = 0
TP.PeakInflow = 0
TP.MaxStage = 0
TP.minstage = 0

TP.MaxVolume = 0
TP.OrigVolume = 0
TP.MinStorage = 0
TP.timestep = 0

TP.numbersteps = 0
TP.InitStage = 0
For i = 1 To MAXSTEPS
        TP.pStage(i) = 0
        TP.Inflow(i) = 0
        TP.outflow(i) = 0
Next i

TP.cw = 0
TP.NumberWeirs = 0
TP.NumberSSD = 0
For i = 1 To SSD_RELATIONSHIPS
        TP.Stage(i) = 0
        TP.Discharge(i) = 0
        TP.Storage(i) = 0
        TP.srfArea(i) = 0
        TP.Weir(1).Stage(i) = 0
        TP.Weir(1).Discharge(i) = 0
        TP.Weir(2).Stage(i) = 0
        TP.Weir(2).Discharge(i) = 0
Next i

For i = 1 To 2
   TP.Weir(i).Label = ""
   TP.Weir(i).Equation = ""
   TP.Weir(i).Type = 0

   TP.Weir(i).Invert = 0
   TP.Weir(i).Diameter = 0
   TP.Weir(i).Width = 0
   TP.Weir(i).C = 0
   TP.Weir(i).n = 0
Next i
End Sub

Sub NewRainfall (TR As RainfallType)
Dim i   As Integer
TR.FileName = "RAINFALL.RNF"
TR.total = 0
TR.numbersteps = 0
TR.duration = 0
TR.timestep = 0
For i = 1 To MAXSTEPS
    TR.rain(i) = 0
Next i
End Sub

Sub NewShedFile (TW As WatershedType)
TW.InfMeth = 0

TW.FileName = "WATRSHED.SHD"
TW.TotalArea = 0
TW.TimeCon = 0
TW.ImpArea = 0
TW.DCIA = 0
TW.AddAbsImp = 0
TW.addabsPerv = 0
TW.MaxInf = 0
TW.HIC = 0
TW.HIO = 0
TW.HK = 0
    
TW.CN = 0
TW.InitAbs = 0

End Sub

Function OpenHydrFile% (TH As HydrographType)
Dim i       As Integer

Input #1, TH.hydrotype
Input #1, TH.KR
Input #1, TH.PeakAttFact
Input #1, TH.hydrosteps
Input #1, TH.timestep
Input #1, TH.TimeCon
Input #1, TH.TotalArea
For i = 1 To TH.hydrosteps
    Input #1, TH.infiltration(i), TH.runoff(i), TH.outflow(i)
       TH.TotalInfiltration = TH.TotalInfiltration + TH.infiltration(i)
       TH.TotalRunoff = TH.TotalRunoff + TH.runoff(i)
       TH.TotalOutflow = TH.TotalOutflow + TH.outflow(i)
       If TH.outflow(i) > TH.PeakOutflow Then TH.PeakOutflow = TH.outflow(i)
Next i
If TH.TotalArea <> 0 Then TH.TotalExcess = TH.TotalRunoff * 60 * TH.timestep / (TH.TotalArea * 3630)
OpenHydrFile% = True
'th.RatCoeff = th.TotalExcess / r.TotalRain
'th.RatCoeff = FIX(th.RatCoeff * 1000) / 1000
'r.TotalRain = FIX(r.TotalRain * 100) / 100
End Function

Function OpenPondFile% (TP As PondType)
Dim i       As Integer
Dim Success As Integer

Input #1, TP.peakstep
Input #1, TP.PeakInflow
Input #1, TP.MaxStage
Input #1, TP.minstage

Input #1, TP.MaxVolume
Input #1, TP.OrigVolume
Input #1, TP.MinStorage
Input #1, TP.timestep

Input #1, TP.numbersteps
Input #1, TP.InitStage
For i = 1 To TP.numbersteps
        Input #1, TP.pStage(i)
        Input #1, TP.Inflow(i)
        Input #1, TP.outflow(i)
Next i

Input #1, TP.NumberSSD
For i = 1 To TP.NumberSSD
        Input #1, TP.Stage(i)
        Input #1, TP.Discharge(i)
        Input #1, TP.Storage(i)
        Input #1, TP.srfArea(i)
Next i

Input #1, TP.cw
Input #1, TP.NumberWeirs
For i = 1 To TP.NumberWeirs
        Success = OpenWeirFile%(TP.Weir(i))
Next i
OpenPondFile% = Success
End Function

Function OpenRainFile% (TR As RainfallType)
Dim i           As Integer

If TR.FileName = "" Then
   OpenRainFile% = False
End If
TR.total = 0

Input #1, TR.Type
Input #1, TR.numbersteps
Input #1, TR.duration
Input #1, TR.timestep
For i = 1 To TR.numbersteps
       Input #1, TR.rain(i)
       TR.total = TR.total + TR.rain(i)
Next i

OpenRainFile% = True
End Function

Function OpenShedFile% (TW As WatershedType)

If TW.FileName = "" Then
   OpenShedFile% = False
End If

Input #1, TW.TotalArea
Input #1, TW.TimeCon
Input #1, TW.ImpArea
Input #1, TW.DCIA
Input #1, TW.AddAbsImp
Input #1, TW.addabsPerv
Input #1, TW.MaxInf
Input #1, TW.InfMeth

Select Case TW.InfMeth
    Case HORTON_INFILTRATION
        Input #1, TW.HIC
        Input #1, TW.HIO
        Input #1, TW.HK
    Case SCS_INFILTRATION
        Input #1, TW.CN
        Input #1, TW.InitAbs
End Select

If TW.DCIA <> 0 Then TW.FracDCIA = (TW.ImpArea / TW.TotalArea) * (TW.DCIA / 100)
If TW.TotalArea <> 0 Then
        TW.FracDCIA = (TW.DCIA / 100) * TW.ImpArea / TW.TotalArea
        TW.FracNDCIA = (TW.ImpArea / TW.TotalArea) * ((100 - TW.DCIA) / 100)
        TW.FracPerv = (TW.TotalArea - TW.ImpArea) / TW.TotalArea
End If

OpenShedFile% = True

End Function

Function OpenUnitFile% (TUH As UnitHydrographType)
Dim i   As Integer
For i = 1 To MAXUNIT: TUH.q(i) = 0: Next i
i = 0
While Not EOF(1)
   i = i + 1: Input #1, TUH.q(i)
Wend
TUH.numbersteps = i
OpenUnitFile% = True
End Function

Function OpenWeirFile% (TW As WeirType)
Dim i       As Integer

Input #1, TW.Label
Input #1, TW.Equation
Input #1, TW.Type

Input #1, TW.Invert
Input #1, TW.Diameter
Input #1, TW.Width
Input #1, TW.C
Input #1, TW.n

Input #1, TW.NumberSSD
For i = 1 To TW.NumberSSD
        Input #1, TW.Stage(i)
        Input #1, TW.Discharge(i)
Next i
OpenWeirFile% = True
End Function

Sub RoundRainfall (places As Integer, TR As RainfallType)

TR.total = 0
For i = 1 To TR.numbersteps
   TR.total = TR.total + TR.rain(i)
   TR.total = Int(TR.total * 10 ^ places + .5) / 10 ^ places
Next i

End Sub

Sub RouteHYDRtoPOND (TH As HydrographType, TP As PondType)

Dim i           As Integer
Dim j           As Integer
Dim k           As Integer
Dim ExitFlag    As Integer

Dim Volume      As Single
Dim tnexts      As Single
Dim flow        As Single

For i = 1 To MAXSTEPS
   TP.Inflow(i) = 0
   TP.outflow(i) = 0
Next i

TP.PeakInflow = 0
For i = 1 To TH.hydrosteps
   TP.Inflow(i) = TH.outflow(i)
   If TP.PeakInflow < TP.Inflow(i) Then TP.PeakInflow = TP.Inflow(i)
Next i

TP.timestep = TH.timestep
If TP.InitStage = 0 Then TP.InitStage = TP.Stage(1)
Volume = getVal!(TP.NumberSSD, TP.InitStage, TP.Stage(), TP.Storage()) * 43560
If Volume < 0 Then Volume = 0

i = 1
TP.MaxVolume = 0
TP.MaxStage = 0
TP.numbersteps = TH.hydrosteps
TP.PeakOutflow = 0
While Not ExitFlag
   For k = 1 To 10
      TP.pStage(i) = getVal!(TP.NumberSSD, Volume / 43560!, TP.Storage(), TP.Stage())
      flow = getVal!(TP.NumberSSD, Volume / 43560, TP.Storage(), TP.Discharge())
      If flow < 0 Then flow = 0
      TP.outflow(i) = flow
      Volume = Volume + (TP.Inflow(i) - TP.outflow(i)) * TH.timestep * 6
      If Volume > TP.MaxVolume Then TP.MaxVolume = Volume
      
      If TP.pStage(i) > TP.MaxStage Then TP.MaxStage = TP.pStage(i)
   Next k
   If TP.outflow(i) > TP.PeakOutflow Then TP.PeakOutflow = TP.outflow(i)
   
   If i = MAXSTEPS - 1 Then ExitFlag = True
   If i > TH.hydrosteps Then
      If TP.outflow(i) < .01 * TP.PeakOutflow Then ExitFlag = True
   End If
   i = i + 1
Wend
TP.numbersteps = i - 1
End Sub

Sub RoutePond (TP As PondType)
Dim i           As Integer
Dim j           As Integer
Dim k           As Integer
Dim ExitFlag    As Integer

Dim Volume      As Single
Dim tnexts      As Single
Dim flow        As Single

For i = 1 To MAXSTEPS
   TP.outflow(i) = 0
   TP.pStage(i) = 0
Next i

TP.PeakInflow = 0
For i = 1 To TP.numbersteps
   If TP.PeakInflow < TP.Inflow(i) Then TP.PeakInflow = TP.Inflow(i)
Next i

If TP.InitStage = 0 Then TP.InitStage = TP.Stage(1)
Volume = getVal!(TP.NumberSSD, TP.InitStage, TP.Stage(), TP.Storage()) * 43560
If Volume < 0 Then Volume = 0

i = 1
TP.MaxVolume = 0
TP.MaxStage = 0
TP.PeakOutflow = 0
While Not ExitFlag
   For k = 1 To 10
      TP.pStage(i) = getVal!(TP.NumberSSD, Volume / 43560!, TP.Storage(), TP.Stage())
      flow = getVal!(TP.NumberSSD, Volume / 43560, TP.Storage(), TP.Discharge())
      If flow < 0 Then flow = 0
      TP.outflow(i) = flow
      Volume = Volume + (TP.Inflow(i) - TP.outflow(i)) * TP.timestep * 6
      If Volume > TP.MaxVolume Then TP.MaxVolume = Volume
      If flow > TP.PeakOutflow Then TP.PeakOutflow = flow
      If TP.pStage(i) > TP.MaxStage Then TP.MaxStage = TP.pStage(i)
   Next k
   i = i + 1
   If i = MAXSTEPS Then ExitFlag = True
   If i > TP.numbersteps And TP.outflow(i) < .01 * TP.PeakOutflow Then ExitFlag = True
Wend
TP.numbersteps = i - 1

End Sub

Function SaveHydrFile% (TH As HydrographType)
Dim i       As Integer
Print #1, TH.hydrotype
Print #1, TH.KR
Print #1, TH.PeakAttFact
Print #1, TH.hydrosteps
Print #1, TH.timestep
Print #1, TH.TimeCon
Print #1, TH.TotalArea
For i = 1 To TH.hydrosteps
    Print #1, TH.infiltration(i), TH.runoff(i), TH.outflow(i)
Next i
SaveHydrFile% = True
End Function

Function SavePondFile% (TP As PondType)
Dim i       As Integer
Dim Success As Integer

Print #1, TP.peakstep
Print #1, TP.PeakInflow
Print #1, TP.MaxStage
Print #1, TP.minstage

Print #1, TP.MaxVolume
Print #1, TP.OrigVolume
Print #1, TP.MinStorage
Print #1, TP.timestep
                                        
Print #1, TP.numbersteps
Print #1, TP.InitStage
For i = 1 To TP.numbersteps
        Print #1, TP.pStage(i)
        Print #1, TP.Inflow(i)
        Print #1, TP.outflow(i)
Next i

Print #1, TP.NumberSSD
For i = 1 To TP.NumberSSD
        Print #1, TP.Stage(i)
        Print #1, TP.Discharge(i)
        Print #1, TP.Storage(i)
        Print #1, TP.srfArea(i)
Next i

Print #1, TP.cw
Print #1, TP.NumberWeirs
For i = 1 To TP.NumberWeirs
        Success = SaveWeirFile%(TP.Weir(i))
Next i
SavePondFile% = Success
End Function

Function SaveRainFile% (TR As RainfallType)
Dim i       As Integer
If TR.timestep = 0 Then
        SaveRainFile% = False
        Exit Function
End If

Print #1, TR.Type
Print #1, TR.numbersteps
Print #1, TR.duration
Print #1, TR.timestep
For i = 1 To TR.numbersteps
     Print #1, TR.rain(i)
Next i

SaveRainFile% = True
End Function

Function SaveShedFile% (TW As WatershedType)

If TW.InfMeth = 0 Then
        SaveShedFile% = False
        Exit Function
End If

Print #1, TW.TotalArea
Print #1, TW.TimeCon
Print #1, TW.ImpArea
Print #1, TW.DCIA
Print #1, TW.AddAbsImp
Print #1, TW.addabsPerv
Print #1, TW.MaxInf
Print #1, TW.InfMeth

Select Case TW.InfMeth
    Case HORTON_INFILTRATION
        Print #1, TW.HIC
        Print #1, TW.HIO
        Print #1, TW.HK
    Case SCS_INFILTRATION
        Print #1, TW.CN
        Print #1, TW.InitAbs
End Select

SaveShedFile% = True
End Function

Function SaveUnitFile% (TUH As UnitHydrographType)
Dim i       As Integer
For i = 1 To TUH.numbersteps
   Print #1, TUH.q(i)
Next i
SaveUnitFile% = True
End Function

Function SaveWeirFile% (TW As WeirType)
Dim i   As Integer
Print #1, TW.Label
Print #1, TW.Equation
Print #1, TW.Type

Print #1, TW.Invert
Print #1, TW.Diameter
Print #1, TW.Width
Print #1, TW.C
Print #1, TW.n

Print #1, TW.NumberSSD
For i = 1 To TW.NumberSSD
        Print #1, TW.Stage(i)
        Print #1, TW.Discharge(i)
Next i
SaveWeirFile% = True
End Function

Sub ScaleRainfall (r As RainfallType)
Dim TotalPrecip         As Single

RoundRainfall 3, r

If r.total = 0 Then Exit Sub
'a$ = "You may scale an existing rainfall file to represent another|"
'a$ = a$ + "total rainfall.|"
'a$ = a$ + "The existing total rainfall is " + STR$(r.TotalRain)
'a$ = a$ + " (in).| The entire rainfall will be scaled to represent the|"
'a$ = a$ + "new total while the shape will be preserved."
'dummy = Alert(4, a$, 10, 10, 18, 70, "Cancel", "Continue", "")
'IF dummy = 0 OR dummy = 1 THEN EXIT SUB

'TotalPrecip = VAL(GetLabel$("", "Input New Total Precipitation (in):"))
If TotalPrecip = 0 Then Exit Sub

For i = 1 To r.numbersteps
        r.rain(i) = r.rain(i) * TotalPrecip / r.total
        r.rain(i) = Int(r.rain(i) * 1000 + .5) / 1000
Next i
r.total = 0
For i = 1 To r.numbersteps
   r.total = r.total + r.rain(i)
Next i
If r.total <> TotalPrecip Then
   r.rain(r.numbersteps) = r.rain(r.numbersteps) + (TotalPrecip - r.total)
   r.rain(r.numbersteps) = Int(r.rain(r.numbersteps) * 1000 + .5) / 1000
End If

End Sub

