' ===========================================================================
'
' SMADA.BAS Copyright (c) 1990 EAGLIN
'
' ===========================================================================
CONST MAXNODES = 10
CONST MAXSTEPS = 260
CONST STORRELATIONS = 10

TYPE watershedtype
        infmeth                     AS SINGLE
        TimeCon                     AS SINGLE
        ImpArea                     AS SINGLE
        DCIA                        AS SINGLE
        AddAbsImp                   AS SINGLE
        AddAbsPerv                  AS SINGLE
        MaxInf                      AS SINGLE
        HIC                         AS SINGLE
        HIO                         AS SINGLE
        HK                          AS SINGLE
        CN                          AS SINGLE
        InitAbs                     AS SINGLE
        FracDCIA                    AS SINGLE
        FracNDCIA                   AS SINGLE
        FracPerv                    AS SINGLE
        SmoothFlag                  AS SINGLE
END TYPE

TYPE pondtype
        peakstep                AS INTEGER
        MaxStage                AS SINGLE
        InitStage               AS SINGLE
        maxvolume               AS SINGLE
        OrigVolume              AS SINGLE
        StageInt                AS SINGLE
        NumStorDisRel           AS INTEGER
        SurfArea                AS SINGLE
        Stage(MAXSTEPS)         AS SINGLE
        StorDis(MAXNODES)       AS SINGLE
        StorDisLimit(MAXNODES)  AS SINGLE
END TYPE



DEFINT A-Y
DECLARE SUB PrintOrderForm ()
DECLARE FUNCTION GetBackStage! (i%, NumBackStages%, BackTime!(), BackStages!())
DECLARE SUB SCSUniform ()
DECLARE SUB UnlinkNode ()
DECLARE SUB SaveBatchData ()
DECLARE FUNCTION routeprint% ()
DECLARE SUB GetBatchNode ()
DECLARE SUB DrawHydrograph (n%)
DECLARE SUB GetHelp (h$)
DECLARE SUB MonoDisplay ()
DECLARE SUB ChangeCurrentDirectory ()
DECLARE SUB colordisplay ()
DECLARE SUB CreateRainFile ()
DECLARE SUB EditRainFile ()
DECLARE SUB GetSTorDisRel ()
DECLARE SUB GetSaveFileName ()
DECLARE SUB MakeDimRainFall ()
DECLARE SUB RetrieveRainFile ()
DECLARE SUB SaveRainFile ()
DECLARE SUB SetupDesktop ()
DECLARE SUB SetupMenu ()
DECLARE SUB ViewHydrograph (NodeFlag%, Steps%, p() AS pondtype)
DECLARE SUB DisplayNodes ()
DECLARE SUB PrintHydrograph ()
DECLARE SUB SaveHydroFile ()
DECLARE SUB RetrieveHydroFile ()
DECLARE SUB PlotHydrograph ()
DECLARE SUB geta ()
DECLARE SUB DeleteFile (FileSpec$)
DECLARE SUB PlotAxis (xmax!, ymax!)
DECLARE SUB PlotSeries (Steps%, Stepsmax%, Valy!(), ymax!)
DECLARE SUB PlotNode ()
DECLARE SUB HyetoPlot (xmax!)
DECLARE SUB ChangeActiveNode ()
DECLARE SUB RainFilePrint ()

'       Watershed Manipulation Subroutines

DECLARE SUB ShedCalc (w AS watershedtype)
DECLARE SUB SCSMethod (w AS watershedtype)
DECLARE SUB CreateShedFile (w AS watershedtype)
DECLARE SUB EditShedFile (w AS watershedtype)
DECLARE SUB GetInfiltration (i, w AS watershedtype)
DECLARE SUB RetrieveShedFile (w AS watershedtype)
DECLARE SUB SantaBarbara (w AS watershedtype)
DECLARE SUB SaveShedFile (w AS watershedtype)
DECLARE SUB SCSMethod (w AS watershedtype)
DECLARE SUB PrintWatershed (w AS watershedtype)
DECLARE SUB GenHydro (w AS watershedtype, p() AS pondtype)

'       Pond Manipulation Routines

DECLARE SUB SaveStorDisRel (pond AS pondtype)
DECLARE SUB RetrieveStorDisRel (pond AS pondtype)
DECLARE SUB MakeStageStorDisRel (pond AS pondtype)
DECLARE SUB EditSTorDisRel (pond AS pondtype)
DECLARE SUB DefinePond (pond AS pondtype)

'       Routing Manipulation Routines

DECLARE SUB DefineNode (p() AS pondtype)
DECLARE SUB LinkNode (p() AS pondtype)
DECLARE SUB DrawNode (n%, p() AS pondtype)
DECLARE SUB PondRoute (ActiveNode%, p() AS pondtype)
DECLARE SUB PrintNode (p() AS pondtype)

DECLARE FUNCTION GetNode% (Message$, FlowToFlag%, ActiveFlag%)
DECLARE FUNCTION ControlEditWindow% (EditFields%, ButtonLine%, h$)
DECLARE FUNCTION FileRetrieve$ (FileSpec$)
DECLARE FUNCTION GetTimeCon% (g%, n!, s!, q!, z!, d!, l!)
DECLARE FUNCTION GetFileCount% (FileSpec$)
DECLARE FUNCTION GetValue! (a!, Message$)
DECLARE FUNCTION GetLabel$ (default$, Message$)
DECLARE FUNCTION GetDiameter! (Diameter!, ActiveNode%)
DECLARE FUNCTION MultAnswer (n%, a$(), title$)
DECLARE FUNCTION GetAnswer (n%, a$(), title$)

'$INCLUDE: 'general.bi'
'$INCLUDE: 'mouse.bi'
'$INCLUDE: 'menu.bi'
'$INCLUDE: 'window.bi'

COMMON SHARED /uitools/ GloMenu           AS MenuMiscType
COMMON SHARED /uitools/ GloTitle()        AS MenuTitleType
COMMON SHARED /uitools/ GloItem()         AS MenuItemType
COMMON SHARED /uitools/ GloWindow()       AS windowType
COMMON SHARED /uitools/ GloButton()       AS buttonType
COMMON SHARED /uitools/ GloEdit()         AS EditFieldType
COMMON SHARED /uitools/ GloStorage        AS WindowStorageType
COMMON SHARED /uitools/ GloWindowStack()  AS INTEGER
COMMON SHARED /uitools/ GloBuffer$()
COMMON SHARED RatCoeff                 AS SINGLE
COMMON SHARED TotalArea                AS SINGLE

DIM SHARED CurrentPath$
DIM SHARED SaveFileName$
DIM SHARED label$
DIM SHARED HydroFile$
DIM SHARED WaterFile$
DIM SHARED RainFile$
DIM SHARED Hydrotype$
DIM SHARED bar$
DIM SHARED Message$

DIM GloTitle(MAXMENU)           AS MenuTitleType
DIM GloItem(MAXMENU, MAXITEM)   AS MenuItemType
DIM GloWindow(MAXWINDOW)        AS windowType
DIM GloButton(MAXBUTTON)        AS buttonType
DIM GloEdit(MaxEditField)       AS EditFieldType
DIM GloWindowStack(MAXWINDOW)   AS INTEGER
DIM GloBuffer$(MAXWINDOW + 1, 2)
DIM SHARED DisplayType          AS INTEGER

' Information generated in hydrograph generate

DIM SHARED KR!
DIM SHARED Runoff(MAXSTEPS)             AS SINGLE
DIM SHARED Infiltration(MAXSTEPS)       AS SINGLE
DIM SHARED Rain(MAXSTEPS)               AS SINGLE
DIM SHARED Outflow(MAXSTEPS)            AS SINGLE
DIM SHARED Excess(MAXSTEPS)             AS SINGLE
DIM SHARED TotalExcess                  AS SINGLE
DIM SHARED TotalRunoff                  AS SINGLE
DIM SHARED TotalInfiltration            AS SINGLE
DIM SHARED TotalOutflow                 AS SINGLE
DIM SHARED TotalRain                    AS SINGLE
DIM SHARED PeakAttFact                  AS SINGLE
DIM SHARED HydroSteps                   AS INTEGER
DIM SHARED PeakOutflow                  AS SINGLE

' Rainfall Information

DIM SHARED NumRainSteps         AS INTEGER
DIM SHARED RainDuration         AS SINGLE
DIM SHARED RainTimeStep         AS INTEGER
DIM SHARED DimFlag              AS INTEGER

' Watershed Information
       DIM w AS watershedtype
' Node Information

DIM SHARED FlowTo(MAXNODES)                     AS INTEGER
DIM SHARED NodeLabel$(MAXNODES)
DIM SHARED NodeInflow(MAXNODES, MAXSTEPS)       AS SINGLE
DIM SHARED NodeOutflow(MAXNODES, MAXSTEPS)      AS SINGLE
DIM SHARED NodeType(MAXNODES)                   AS INTEGER
DIM SHARED NodeExist(MAXNODES)                  AS INTEGER
DIM SHARED PeakNodeOutflow                      AS SINGLE
DIM SHARED ActiveNode                           AS INTEGER
DIM SHARED Length(MAXNODES)                     AS SINGLE
DIM SHARED Diameter(MAXNODES)                   AS SINGLE
DIM SHARED Bas(MAXNODES)                        AS SINGLE
DIM SHARED velocity(MAXNODES)                   AS SINGLE
DIM SHARED Depth(MAXNODES)                      AS SINGLE
DIM SHARED NodeGeometry(MAXNODES)               AS INTEGER
DIM SHARED Slope(MAXNODES)                      AS SINGLE
DIM SHARED SideSlope(MAXNODES)                  AS SINGLE

' Pond Information
        DIM p(MAXNODES) AS pondtype
        DIM pond        AS pondtype

bar$ = "        " + STRING$(66, "-")
DIM SHARED help$
DIM SHARED rootdir$
rootdir$ = CURDIR$
help$ = CURDIR$ + "\" + "smhelp.hlp"

    ' =======================================================================
    ' Initialize Demo
    ' =======================================================================

    MenuInit
    WindowInit
    MouseShow
    colordisplay

         a$ = "SMADA 4.0   Release date 10/28/91|"
    a$ = a$ + "Stormwater Management and Design Aid|"
    a$ = a$ + "copyright (c) 1991|"
    a$ = a$ + "Written by|"
    a$ = a$ + "|"
    a$ = a$ + "Ron Eaglin and Marty Wanielista|"
    a$ = a$ + "Program Team: Greg Harper|"
    a$ = a$ + "                : Marc Gauthier"
    
        a = Alert(4, a$, 10, 10, 19, 70, "", "Continue", "")
        colordisplay

    ' =======================================================================
    ' Main Loop : Stay in loop until DemoFinished set to TRUE
    ' =======================================================================
  
    DemoFinished = FALSE

    WHILE NOT DemoFinished
        kbd$ = MenuInkey$
        WHILE MenuCheck(2)
            GOSUB MenuTrap
        MenuOn
        WEND
    WEND

    ' =======================================================================
    ' End Program
    ' =======================================================================

    MouseHide
    COLOR 15, 0
    CLS
    END



' ===========================================================================
' If a menu event occured, call the proper demo, or if Exit, set demoFinished
' ===========================================================================

MenuTrap:
    menu = MenuCheck(0)
    item = MenuCheck(1)
   
    SELECT CASE menu
        CASE 1
            SELECT CASE item
                CASE 1: GetHelp help$
                CASE 2: ChangeCurrentDirectory
                CASE 3: LPRINT CHR$(12)
                CASE 5: DemoFinished = TRUE
            END SELECT
        CASE 2
            SELECT CASE item
                CASE 1: CreateShedFile w
                CASE 2: EditShedFile w
                CASE 3: RetrieveShedFile w
                CASE 4: SaveShedFile w
                CASE 5: DeleteFile ("*.shd")
                CASE 6: PrintWatershed w
            END SELECT
        CASE 3
            SELECT CASE item
                CASE 1: DimFlag = 0: CreateRainFile
                CASE 2: EditRainFile
                CASE 3: RetrieveRainFile
                CASE 4: SaveRainFile
                CASE 5: DeleteFile ("*.rnf")
                CASE 6: DimFlag = 1: CreateRainFile
                CASE 7: RainFilePrint
            END SELECT
        CASE 4
            SELECT CASE item
                CASE 1: GenHydro w, p()
                CASE 2: ViewHydrograph FALSE, HydroSteps, p()
                CASE 3: RetrieveHydroFile
                CASE 4: SaveHydroFile
                CASE 5: PlotHydrograph
                CASE 6: PrintHydrograph
            END SELECT
        CASE 5
            SELECT CASE item
                CASE 1: DefinePond pond
                CASE 2: EditSTorDisRel pond
                CASE 3: RetrieveStorDisRel pond
                CASE 4: SaveStorDisRel pond
            END SELECT
        CASE 6
            SELECT CASE item
                CASE 1: DefineNode p()
                CASE 2: LinkNode p()
                CASE 3: SaveBatchData
                CASE 4: DisplayNodes
                CASE 5: PlotNode
                CASE 6: PrintNode p()
                CASE 7: ViewHydrograph TRUE, 90, p()
                CASE 8: UnlinkNode
            END SELECT
        CASE 7
            SELECT CASE item
                CASE 1: PrintOrderForm
                        colordisplay
             
            END SELECT
        MenuOn
        CASE ELSE
    END SELECT
RETURN

DEFINT Z
SUB ChangeCurrentDirectory

OrigDir$ = CURDIR$
Origdrive$ = LEFT$(CURDIR$, 2)

currentpathtemp$ = GetLabel$(OrigDir$, "Choose New Directory:")
IF LEN(currentpathtemp$) < 2 THEN currentpathtemp$ = OrigDir$
   ON LOCAL ERROR GOTO handler
   junk = Alert(4, "You Chose " + currentpathtemp$, 10, 10, 12, 70, "Continue", "Cancel", "")
   SELECT CASE junk
   CASE 2
        EXIT SUB
   CASE 1
        IF INSTR(1, currentpathtemp$, ":") = 2 THEN
                IF LEN(currentpathtemp$) = 2 THEN currentpathtemp$ = currentpathtemp$ + "\"
                newdrive$ = LEFT$(currentpathtemp$, 2)
                CHDRIVE newdrive$
                CHDIR currentpathtemp$
        ELSE
                CHDIR currentpathtemp$
        END IF
   END SELECT

handler:
    IF ERR = 76 THEN
        junk = Alert(4, "The Directory Could Not Be Found", 10, 10, 12, 70, "OK", "", "")
        CHDRIVE Origdrive$
        CHDIR OrigDir$
        EXIT SUB
    END IF



END SUB

SUB colordisplay
    DisplayType = TRUE
    MouseHide
    SetupMenu
    SetupDesktop
    MenuShow
    MouseShow
END SUB

SUB CreateRainFile

WindowOpen 1, 6, 5, 12, 75, 0, 7, 0, 7, 15, FALSE, FALSE, FALSE, TRUE, 1, "CREATE RAINFALL ARRAY"

windowlocate 2, 2
windowprint 2, "Time step of Rainfall Increments (minutes):"
windowlocate 4, 2
windowprint 2, "Total Duration of Rainfall event (hours):"

editfieldopen 1, "", 2, 50, 14, 1, 10, 10
editfieldopen 2, "", 4, 50, 14, 1, 10, 10

IF ControlEditWindow(2, 7, help$) = FALSE THEN EXIT SUB

RainTimeStep = VAL(EditFieldInquire(1))
RainDuration = VAL(EditFieldInquire(2))

IF RainTimeStep <> 0 THEN NumRainSteps = RainDuration * 60 / RainTimeStep + .5
WindowClose 1

FOR i = 1 TO MAXSTEPS
        Rain(i) = 0
NEXT i

IF DimFlag = 1 THEN MakeDimRainFall

IF NumRainSteps > 96 THEN
        a$ = "You have exceeded 96 Rainfall increments|"
        junk = Alert(4, a$, 7, 9, 12, 69, "", "Continue", "")
        EXIT SUB
END IF

EditRainFile

END SUB

SUB CreateShedFile (w AS watershedtype)

a$ = "You Must Choose a method for the prediction of infiltration.|"
a$ = a$ + "The methods available are the Horton Infiltration method and|"
a$ = a$ + "the SCS - Curve Number Method"

w.infmeth = Alert(4, a$, 6, 5, 10, 75, "Cancel", "Horton", "SCS-CN")

IF w.infmeth = 1 THEN
        w.infmeth = 0
        EXIT SUB
ELSE
        EditShedFile w
END IF

END SUB

DEFSNG Z
SUB DefineNode (p() AS pondtype)
DIM a$(MAXNODES)
DIM b$(5)

b$(1) = "Watershed Node"
b$(2) = "Pond Node"
b$(3) = "Junction Node"
b$(4) = "Final Outlet"
b$(5) = "Batch Data File"
NodeType = GetAnswer(5, b$(), "Choose Node Type")

IF NodeType = 0 THEN EXIT SUB

FOR i = 1 TO MAXNODES
        a$(i) = "Node " + STR$(i):
        IF NodeExist(i) = TRUE THEN a$(i) = a$(i) + "  " + NodeLabel$(i)
NEXT i

NodeNumber = MultAnswer(MAXNODES, a$(), "Choose Number to assign to node")
IF NodeExist(NodeNumber) = TRUE THEN junk = Alert(4, "Node is already defined, redefine?", 5, 10, 8, 70, "Yes", "No", "")
IF junk = 2 THEN EXIT SUB
IF junk = 1 THEN FOR i = 1 TO MAXSTEPS: NodeInflow(NodeNumber, i) = 0: NodeOutflow(ActiveNode, i) = 0: NEXT i

IF NodeNumber = 0 THEN EXIT SUB
ActiveNode = NodeNumber

Message$ = "Enter a Name for Node # " + STR$(NodeNumber)
NodeLabel$(NodeNumber) = GetLabel$(b$(NodeType), Message$)

IF NodeType = 5 THEN GetBatchNode

IF NodeType = 1 THEN
        RetrieveHydroFile
        IF HydroSteps = 0 THEN EXIT SUB

        FOR i = 1 TO HydroSteps
                NodeInflow(NodeNumber, i) = Outflow(i)
        NEXT i
END IF

NodeExist(NodeNumber) = TRUE
NodeType(NodeNumber) = NodeType
IF NodeType = 2 THEN RetrieveStorDisRel p(ActiveNode)
colordisplay

END SUB

SUB DefinePond (pond AS pondtype)
DIM a$(5)
a$(1) = "Storage Discharge Relation"
a$(2) = "Outlet Weir Relationship"

temppondtype = GetAnswer(2, a$(), "Define Discharge Method")
IF temppondtype = 0 THEN EXIT SUB
IF temppondtype = 1 THEN EditSTorDisRel pond

IF pondtype = 2 THEN
        MakeStageStorDisRel pond
        EditSTorDisRel pond
END IF


END SUB

SUB DeleteFile (FileSpec$)
RFile$ = FileRetrieve(FileSpec$)
IF RFile$ <> "" THEN
        a$ = RFile$ + " Will be Deleted"
        junk = Alert(4, a$, 10, 20, 15, 60, "Delete", "Cancel", "")
        IF junk = 1 THEN KILL RFile$
END IF
END SUB

SUB DisplayNodes
MenuOff
a$ = "Defined Nodes| |"
FOR i = 1 TO MAXNODES
        IF NodeExist(i) = TRUE THEN

                a$ = a$ + "Node: " + STR$(i) + " (" + NodeLabel$(i) + ")"
                IF NodeExist(FlowTo(i)) = TRUE THEN
                        a$ = a$ + "   Flows to Node:" + STR$(FlowTo(i)) + " (" + NodeLabel$(FlowTo(i)) + ")"
                ELSE
                        IF FlowTo(i) <> 0 THEN
                                a$ = a$ + "   Flows to Node:" + STR$(FlowTo(i)) + " (Undefined)"
                        ELSE
                                a$ = a$ + "    is not linked"
                        END IF
                END IF
                a$ = a$ + "|"
        END IF
NEXT i
junk = Alert(4, a$, 5, 10, 21, 70, "", "", "")
END SUB

SUB DrawHydrograph (n)
DIM TotalRunoff2 AS SINGLE
DIM TotalOutflow2 AS SINGLE
WindowCls
windowlocate 1, 2
windowprint 2, "  Time    Rainfall    Infiltration        Hydrographs in cfs"
windowlocate 2, 2
windowprint 2, " Hours     (in)          (in)         Instantaneous      Outflow"
windowline 3
FOR i = 1 TO 16
windowlocate i + 3, 2
windowprint 1, STR$(INT(RainTimeStep * (i + n) * 5 / 3) / 100)
windowlocate i + 3, 12
windowprint 1, STR$(Rain(i + n))
windowlocate i + 3, 25
windowprint 1, STR$(Infiltration(i + n))
windowlocate i + 3, 42
windowprint 1, STR$(Runoff(i + n))
windowlocate i + 3, 58
windowprint 1, STR$(Outflow(i + n))
NEXT i
windowline 19
windowlocate 20, 2
windowprint 2, "Total ---"
windowlocate 20, 12
windowprint 2, STR$(INT(TotalRain * 100) / 100)
windowlocate 20, 42
TotalRunoff2 = TotalRunoff * RainTimeStep * 60
windowprint 2, STR$(TotalRunoff2) + " cf"
windowlocate 20, 58
TotalOutflow2 = TotalOutflow * RainTimeStep * 60
windowprint 2, STR$(TotalOutflow2) + " cf"
windowlocate 21, 2
windowprint -2, "Rational Coefficient = " + STR$(RatCoeff)
windowlocate 21, 46
windowprint -2, "Peak Flow = " + STR$(PeakOutflow) + " cfs"
END SUB

SUB DrawNode (n%, p() AS pondtype)
WindowCls
windowlocate 1, 2
windowprint 2, "            Time        Rain        Node Inflow        Node Outflow"
windowlocate 2, 2
windowprint 2, "            Hours        (in)         (cfs)              (cfs)"
windowline 3
FOR i = 1 TO 16
NodeInflow(ActiveNode, i + n) = FIX(NodeInflow(ActiveNode, i + n) * 1000) / 1000
NodeOutflow(ActiveNode, i + n) = FIX(NodeOutflow(ActiveNode, i + n) * 1000) / 1000
windowlocate i + 3, 12
windowprint 1, STR$(INT(RainTimeStep * (i + n) * 5 / 3) / 100)
windowlocate i + 3, 24
windowprint 1, STR$(Rain(i + n))
windowlocate i + 3, 36
windowprint 1, STR$(NodeInflow(ActiveNode, (i + n)))
windowlocate i + 3, 54
windowprint 1, STR$(NodeOutflow(ActiveNode, (i + n)))
NEXT i
windowline 19
SELECT CASE NodeType(ActiveNode)
        CASE 1, 3, 5
                SELECT CASE NodeType(ActiveNode)
                       CASE 3: b$ = "Node :" + STR$(ActiveNode) + "   Junction Node"
                       CASE 5: b$ = "Node :" + STR$(ActiveNode) + "   Batch Node"
                       CASE 1: b$ = "Node :" + STR$(ActiveNode) + "   Watershed Node"
                END SELECT
                SELECT CASE NodeGeometry(ActiveNode)
                        CASE 1
                                b$ = b$ + "   Pipe Outlet"
                                a$ = "Length :" + STR$(Length(ActiveNode))
                                a$ = a$ + " ft   Diameter: " + STR$(Diameter(ActiveNode)) + " ft"
                                a$ = a$ + "   Velocity (full flow) = " + STR$(velocity(ActiveNode)) + " fps"
                        CASE 2
                                 b$ = b$ + " Length :" + STR$(Length(ActiveNode)) + " ft"
                                 b$ = b$ + "   Velocity:" + STR$(velocity(ActiveNode)) + "  fps"
                                 a$ = "Depth:" + STR$(Depth(ActiveNode))
                                 a$ = a$ + " ft   Bottom Width:" + STR$(Bas(ActiveNode))
                                 a$ = a$ + " ft   Side slope:" + STR$(SideSlope(ActiveNode))
                                 a$ = a$ + " v/h"
                        END SELECT
        CASE 2
                 b$ = "Node :" + STR$(ActiveNode) + "   Pond Node"
                 x! = FIX(p(ActiveNode).maxvolume * 1000 / 43560) / 1000
                 a$ = "Maximum Volume = " + STR$(x!) + " ac-ft  Maximum Stage = " + STR$(p(ActiveNode).MaxStage) + " ft"
        CASE 4
                 b$ = "Node :" + STR$(ActiveNode) + "    Outlet Node"
END SELECT
windowlocate 20, 2
windowprint -1, b$
windowlocate 21, 2
windowprint -1, a$

END SUB

DEFINT Z
SUB EditRainFile
DIM a$(97)

MenuOff

IF RainTimeStep = 0 THEN EXIT SUB


TotalRain = 0
FOR i = 1 TO NumRainSteps
        IF Rain(i) <> 0 THEN a$(i) = STR$(Rain(i)) ELSE a$(i) = ""
        TotalRain = TotalRain + Rain(i)
NEXT i

WindowOpen 1, 3, 3, 24, 75, 0, 7, 0, 7, 15, FALSE, FALSE, FALSE, TRUE, 1, "Rainfall Edit"

        windowlocate 1, 1
        windowprint 2, "Time  Rain  Time  Rain   Time  Rain   Time  Rain   Time  Rain  Time  Rain"
        windowlocate 2, 1
        windowprint 2, "min   inch   min  inch   min   inch   min   inch   min   inch  min   inch"

FOR i = 1 TO NumRainSteps
        RainTime = RaintTime + i * RainTimeStep
        RainTime$ = LTRIM$(STR$(RainTime))
        GOSUB getpos
        windowlocate posy, posx
        windowprint 2, RainTime$
        windowlocate posy, posx + 5
        windowprint -1, a$(i)
NEXT i

        ButtonOpen 1, 1, "Complete", 22, 59, 0, 0, 1
        windowlocate 22, 30
        windowprint -1, "Total Rain"
        ExitFlag = FALSE
        i = 1
        GOSUB getpos
        editfieldopen 1, STR$(Rain(i)), posy, posx + 5, 14, 1, 5, 5


WHILE NOT ExitFlag
        TotalRain = CSNG(CINT(TotalRain * 100) / 100)
        windowlocate 22, 42
        windowprint -1, STR$(TotalRain) + " in     "
        WindowDo currButton, 1
        SELECT CASE Dialog(0)
               
                CASE 1, 9
                        ExitFlag = TRUE
                CASE 10
                        IF i > 1 THEN j = -1 ELSE j = 0
                        GOSUB newpos
                CASE 11, 6
                        IF i < NumRainSteps THEN j = 1 ELSE j = 0: ExitFlag = TRUE
                        GOSUB newpos
        END SELECT
        WEND

lrain! = Rain(i)
Rain(i) = VAL(EditFieldInquire(1))
TotalRain = TotalRain + Rain(i) - lrain!

WindowClose 1

EXIT SUB
getpos:
        IF i < 20 THEN posy = i + 2: posx = 1
        IF i >= 20 AND i < 39 THEN posy = i - 17: posx = 14
        IF i >= 39 AND i < 58 THEN posy = i - 36: posx = 27
        IF i >= 58 AND i < 77 THEN posy = i - 55: posx = 40
        IF i >= 77 AND i < 96 THEN posy = i - 74: posx = 53
        IF i >= 96 THEN posy = i - 93: posx = 64
RETURN
newpos:
        lrain! = Rain(i)
        Rain(i) = VAL(EditFieldInquire(1))
        EditFieldClose 1
        TotalRain = TotalRain + Rain(i) - lrain!
        GOSUB getpos
        windowlocate posy, posx + 5
        IF Rain(i) = 0 THEN a$(i) = "" ELSE a$(i) = STR$(Rain(i))
        windowprint -1, a$(i)
        i = i + j
        GOSUB getpos
        IF Rain(i) = 0 THEN a$(i) = "" ELSE a$(i) = STR$(Rain(i))
        editfieldopen 1, a$(i), posy, posx + 5, 14, 1, 5, 5
RETURN
END SUB

SUB EditShedFile (w AS watershedtype)
DIM a$(10)
MenuOff

IF w.infmeth = 0 THEN EXIT SUB

IF TotalArea <> 0 THEN a$(1) = STR$(TotalArea) ELSE a$(1) = ""
IF w.TimeCon <> 0 THEN a$(2) = STR$(w.TimeCon) ELSE a$(2) = ""
IF w.ImpArea <> 0 THEN a$(3) = STR$(w.ImpArea) ELSE a$(3) = ""
IF w.DCIA <> 0 THEN a$(4) = STR$(w.DCIA) ELSE a$(4) = ""
IF w.AddAbsImp <> 0 THEN a$(5) = STR$(w.AddAbsImp) ELSE a$(5) = ""
IF w.AddAbsPerv <> 0 THEN a$(6) = STR$(w.AddAbsPerv) ELSE a$(6) = ""
IF w.MaxInf <> 0 THEN a$(7) = STR$(w.MaxInf) ELSE a$(7) = ""

WindowOpen 1, 5, 5, 18, 75, 0, 7, 0, 7, 15, FALSE, FALSE, FALSE, TRUE, 1, "Watershed Edit"

windowlocate 1, 5
windowprint 1, "Drainage area (acres)"
editfieldopen 1, a$(1), 1, 55, 14, 1, 10, 10
windowlocate 2, 5
windowprint 1, "Time of Concentration (min)"
editfieldopen 2, a$(2), 2, 55, 14, 1, 10, 10
windowlocate 3, 5
windowprint 1, "Impervious Watershed Area (acres)"
editfieldopen 3, a$(3), 3, 55, 14, 1, 10, 10
windowlocate 4, 5
windowprint 1, "Percent of Impervious Directly Connected (%)"
editfieldopen 4, a$(4), 4, 55, 14, 1, 10, 10
windowlocate 5, 5
windowprint 1, "Additional Abstraction for Impervious Area (in)"
editfieldopen 5, a$(5), 5, 55, 14, 1, 10, 10
windowlocate 6, 5
windowprint 1, "Additional Abstraction for Pervious Area (in)"
editfieldopen 6, a$(6), 6, 55, 14, 1, 10, 10
windowlocate 7, 5
windowprint 1, "Maximum Infiltration Capacity (in)"
editfieldopen 7, a$(7), 8, 55, 14, 1, 10, 10
windowlocate 8, 5
windowprint 1, "Input 999 if No Limit Exists"

IF w.infmeth = 2 THEN
        IF w.HIC <> 0 THEN a$(8) = STR$(w.HIC) ELSE a$(8) = ""
        IF w.HIO <> 0 THEN a$(9) = STR$(w.HIO) ELSE a$(9) = ""
        IF w.HK <> 0 THEN a$(10) = STR$(w.HK) ELSE a$(10) = ""
        MaxField = 10
        windowlocate 10, 5
        windowprint 1, "Limiting Infiltration Rate (in/hr)"
        editfieldopen 8, a$(8), 10, 55, 14, 1, 10, 10
        windowlocate 11, 5
        windowprint 1, "Initial Infiltration Rate (in/hr)"
        editfieldopen 9, a$(9), 11, 55, 14, 1, 10, 10
        windowlocate 12, 5
        windowprint 1, "Horton Depletion Coefficient (/hr)"
        editfieldopen 10, a$(10), 12, 55, 14, 1, 10, 10
        EditFields = 10
END IF
IF w.infmeth = 3 THEN
        IF w.CN <> 0 THEN a$(8) = STR$(w.CN) ELSE a$(8) = ""
        IF w.InitAbs <> 0 THEN a$(9) = STR$(w.InitAbs) ELSE a$(9) = ".2"
        MaxField = 9
        windowlocate 10, 5
        windowprint 1, "SCS Curve Number (for Pervious)"
        editfieldopen 8, a$(8), 10, 55, 14, 1, 10, 10
        windowlocate 11, 5
        windowprint 1, "SCS Abstraction Factor"
        editfieldopen 9, a$(9), 11, 55, 14, 1, 10, 10
        EditFields = 9
END IF
IF ControlEditWindow(EditFields, 14, help$) = FALSE THEN EXIT SUB

TotalArea = VAL(EditFieldInquire(1))
w.TimeCon = VAL(EditFieldInquire(2))
w.ImpArea = VAL(EditFieldInquire(3))
w.DCIA = VAL(EditFieldInquire(4))
w.AddAbsImp = VAL(EditFieldInquire(5))
w.AddAbsPerv = VAL(EditFieldInquire(6))
w.MaxInf = VAL(EditFieldInquire(7))

SELECT CASE w.infmeth
CASE 2
        w.HIC = VAL(EditFieldInquire(8))
        w.HIO = VAL(EditFieldInquire(9))
        w.HK = VAL(EditFieldInquire(10))
CASE 3
        w.CN = VAL(EditFieldInquire(8))
        w.InitAbs = VAL(EditFieldInquire(9))

END SELECT

WindowClose 1

ShedCalc w

END SUB

DEFSNG Z
SUB EditSTorDisRel (pond AS pondtype)

IF pond.InitStage = 0 THEN pond.InitStage = VAL(GetLabel$("", "Input minimum stage in relationship (ft)"))
IF pond.NumStorDisRel = 0 THEN pond.NumStorDisRel = VAL(GetLabel$("", "How Many Relationships"))
IF pond.StageInt = 0 THEN pond.StageInt = VAL(GetLabel$("1", "Input Increment of Relationships (ft)"))

WindowOpen 1, 4, 5, 8 + pond.NumStorDisRel, 75, 0, 7, 0, 7, 15, FALSE, FALSE, FALSE, TRUE, 1, ""

windowlocate 1, 4
windowprint -1, "Stage(ft)       Storage (acre-feet)       Discharge (cfs)"
windowline 2
FOR i = 1 TO pond.NumStorDisRel
        windowlocate 2 + i, 6
        windowprint -1, STR$(pond.InitStage + (i - 1) * pond.StageInt)
        IF pond.StorDisLimit(i) = 0 THEN a$ = "" ELSE a$ = STR$(pond.StorDisLimit(i))
        IF pond.StorDis(i) = 0 THEN b$ = "" ELSE b$ = STR$(pond.StorDis(i))
        editfieldopen 1 + (i - 1) * 2, a$, i + 2, 25, 14, 1, 10, 10
        editfieldopen 2 + (i - 1) * 2, b$, i + 2, 45, 14, 1, 10, 10
NEXT i

StorFlag = ControlEditWindow(pond.NumStorDisRel * 2, 5 + pond.NumStorDisRel, help$)

FOR i = 1 TO pond.NumStorDisRel
        pond.StorDisLimit(i) = VAL(EditFieldInquire(1 + (i - 1) * 2))
        pond.StorDis(i) = VAL(EditFieldInquire(2 + (i - 1) * 2))
NEXT i
WindowClose 1

pond.OrigVolume = GetValue(0, "Input initial volume in pond (acre-feet)")

END SUB

FUNCTION FileRetrieve$ (FileSpec$)
        delimit = INSTR(FileSpec$, ".")

    IF delimit THEN
        fileName$ = LEFT$(FileSpec$, delimit - 1)
        fileExt$ = RIGHT$(FileSpec$, LEN(FileSpec$) - (delimit))
    ELSE
        fileName$ = FileSpec$
        fileExt$ = ""
    END IF

    IF LEN(FileSpec$) = 0 OR LEN(fileName$) > 8 OR LEN(fileExt$) > 3 THEN
        EXIT FUNCTION
    END IF

    FileCount = GetFileCount(FileSpec$)

    IF FileCount THEN

        REDIM FileList$(FileCount)

    ELSE

        junk = Alert(4, "No match to your file specification could be found.", 10, 10, 12, 70, "OK", "", "")
        EXIT FUNCTION
    END IF

    FileList$(1) = DIR$(FileSpec$)

    FOR Indx = 2 TO FileCount
        FileList$(Indx) = DIR$
    NEXT Indx

    x = ListBox(FileList$(), UBOUND(FileList$))

    FileRetrieve$ = FileList$(x)
    WindowClose 1
END FUNCTION

SUB GenHydro (w AS watershedtype, p() AS pondtype)
REDIM a$(5)

a$(1) = "Santa Barbara"
a$(2) = "Discrete Unit Convolution"
a$(3) = "Rational"
a$(4) = "SCS 484 Curvilinear Unit"
a$(5) = "SCS 256 Curvilinear Unit"
choice = GetAnswer(5, a$(), "Choose Generation Method:")
IF choice = 0 THEN EXIT SUB

FOR i = 1 TO MAXSTEPS
        Outflow(i) = 0
        Runoff(i) = 0
NEXT i

HydroSteps = 0
w.SmoothFlag = FALSE
SELECT CASE choice
        CASE 0: EXIT SUB
        CASE 1: SantaBarbara w
        CASE 2: PeakAttFact = 0: SCSMethod w
        CASE 3: PeakAttFact = 645: SCSMethod w
        CASE 4: PeakAttFact = 484: w.SmoothFlag = TRUE: SCSMethod w
        CASE 5: PeakAttFact = 256: w.SmoothFlag = TRUE: SCSMethod w
END SELECT

IF HydroSteps = 0 THEN EXIT SUB
TotalRunoff = 0
TotalOutflow = 0
FOR i = 1 TO HydroSteps
        TotalOutflow = TotalOutflow + Outflow(i)
        TotalRunoff = TotalRunoff + Runoff(i)
        IF Outflow(i) < 0 THEN Outflow(i) = 0
        IF Outflow(i) > PeakOutflow THEN PeakOutflow = Outflow(i)
NEXT i

HydroFile$ = "Unnamed " + a$(choice)
IF TotalRain <> 0 THEN RatCoeff = TotalExcess / TotalRain
RatCoeff = CSNG(CINT(RatCoeff * 100) + .5) / 100
TotalRain = CSNG(CINT(TotalRain * 100) + .5) / 100

ViewHydrograph FALSE, HydroSteps, p()

END SUB

FUNCTION GetBackStage! (i, NumBackStages, BackTime!(), BackStages!())

T! = i * RainTimeStep / 60
BackStage! = 0
        IF T! > BackTime!(NumBackStages) THEN
                GetBackStage! = BackStages!(NumBackStages)
                EXIT FUNCTION
        END IF

FOR j = NumBackStages - 1 TO 1 STEP -1
      
        IF T! > BackTime!(j) THEN
                s! = (BackStages!(j + 1) - BackStages!(j)) / (BackTime!(j + 1) - BackTime!(j))
                BackStage! = BackStages!(j) + s! * (T! - BackTime!(j))
                GetBackStage! = BackStage!
                EXIT FUNCTION
        END IF
NEXT j

GetBackStage! = 0

END FUNCTION

SUB GetBatchNode
a$ = "A Batch Node is Data which is input by hand to a file|"
a$ = a$ + "on a disk which has an extension of .DAT   For more|"
a$ = a$ + "information on using batch data nodes read the message|"
a$ = a$ + "in the on-line help under Batch-Data"

junk = Alert(4, a$, 5, 5, 11, 75, "Continue", "Cancel", "")
IF RainTimeStep = 0 THEN RainTimeStep = VAL(GetLabel$("", "What is the Time step?"))
IF junk <> 1 THEN EXIT SUB
RFile$ = FileRetrieve$("*.dat")
IF RFile$ <> "" THEN
        OPEN RFile$ FOR INPUT AS #1
        i = 1
        WHILE NOT EOF(1) AND i < MAXSTEPS
                INPUT #1, NodeInflow(ActiveNode, i)
                i = i + 1
        WEND
        CLOSE #1
        HydroSteps = i - 1
END IF

END SUB

FUNCTION GetDiameter! (Diameter!, ActiveNode%)
DIM a$(2)
a$ = "Required Diameter =" + STR$(Diameter!) + " in"
a$(1) = "Use Standard Diameter"
a$(2) = "Input Custom Diameter"
Diachoice = GetAnswer(2, a$(), a$)
IF Diachoice = 1 THEN
        WHILE Diameter! > standardPipe
                standardPipe = 9 + 3 * i
                IF i > 7 THEN standardPipe = 30 + (i - 7) * 6
                IF i > 12 THEN standardPipe = 60 + (i - 12) * 12
                i = i + 1
        WEND
        GetDiameter! = standardPipe
ELSE
        GetDiameter! = VAL(GetLabel$("", "Input Diameter in Inches"))
END IF
END FUNCTION

DEFINT Z
FUNCTION GetFileCount (FileSpec$)
    count = 0
    fileName$ = DIR$(FileSpec$)
    DO WHILE fileName$ <> ""
        count = count + 1
        fileName$ = DIR$
    LOOP
    GetFileCount = count
END FUNCTION

DEFSNG A-H, J-Z
SUB GetInfiltration (i, w AS watershedtype)

STATIC TotalPervRain    AS SINGLE
STATIC cuminf           AS SINGLE
STATIC PervAbs          AS SINGLE
STATIC CumPervExcess    AS SINGLE
STATIC LastCumPervRunoff AS SINGLE
STATIC CumPervRunoff    AS SINGLE

DIM ImpExcess           AS SINGLE
DIM PervExcess          AS SINGLE
DIM ImpAbs              AS SINGLE
DIM ImpToPervExcess     AS SINGLE
DIM j                   AS INTEGER
DIM DummyCumInf         AS SINGLE
DIM PervRunoff          AS SINGLE
DIM ImpRunoff           AS SINGLE

IF i = 1 THEN
        TotalExcess = 0
        TotalRain = 0
        TotalPervRain = 0
        cuminf = 0
        TotalRunoff = 0
        TotalInfiltration = 0
        CumPervExcess = 0
        CumPervRunoff = 0
        LastCumPervRunoff = 0
END IF

TotalRain = TotalRain + Rain(i)
TIME = i * RainTimeStep

IF w.AddAbsImp > TotalRain THEN
        ImpExcess = 0
        ImpAbs = TotalRain
ELSE
        ImpExcess = Rain(i) - w.AddAbsImp
        ImpAbs = w.AddAbsImp
        IF ImpExcess < 0 THEN ImpExcess = 0
END IF

ImpToPervExcess = w.FracNDCIA * ImpExcess
ImpRunoff = ImpExcess
PervRain = Rain(i) + ImpToPervExcess
TotalPervRain = PervRain + TotalPervRain

IF w.AddAbsPerv > TotalPervRain THEN
        PervExcess = 0
        PervAbs = TotalPervRain
ELSE
        PervExcess = PervRain + PervAbs - w.AddAbsPerv
        PervAbs = w.AddAbsPerv
END IF
CumPervExcess = CumPervExcess + PervExcess

IF w.infmeth = 2 THEN
        ExitFlag = FALSE
        j = 0
        WHILE NOT ExitFlag
                DummyCumInf = ((w.HIC * j * RainTimeStep) / 60 + (w.HIO - w.HIC) * (1 - EXP(-w.HK * j * RainTimeStep / 60))) / w.HK
                IF cuminf > DummyCumInf THEN
                        j = j + 1
                ELSE
                        DummyTime = j * RainTimeStep / 60
                        ExitFlag = TRUE
                END IF
        WEND
        PotInf = (w.HIC + (w.HIO - w.HIC) * EXP(-w.HK * DummyTime)) * RainTimeStep / 60

        IF cuminf < w.MaxInf THEN
                IF PervExcess > PotInf THEN
                        PervRunoff = PervExcess - PotInf
                        Infiltration(i) = PotInf
                ELSE
                        PervRunoff = 0
                        Infiltration(i) = PervExcess
                END IF
        ELSE
                Infiltration(i) = 0
        END IF
END IF

IF w.infmeth = 3 THEN
        SPrime = 1000 / w.CN - 10
        AbsFact = w.InitAbs * SPrime
        IF CumPervExcess > AbsFact THEN
                CumPervRunoff = ((CumPervExcess - AbsFact) ^ 2) / (CumPervExcess + (1 - w.InitAbs) * SPrime)
                PervRunoff = CumPervRunoff - LastCumPervRunoff
                LastCumPervRunoff = CumPervRunoff
                IF cuminf < w.MaxInf THEN
                        Infiltration(i) = PervExcess - PervRunoff
                ELSE
                        Infiltration(i) = 0
                        PervRunoff = PervExcess
                END IF
        ELSE
                IF cuminf < w.MaxInf THEN
                        PervRunoff = 0
                        Infiltration(i) = PervExcess
                ELSE
                        PervRunoff = PervExcess
                        Infiltration(i) = 0
                END IF
        END IF
END IF

cuminf = cuminf + Infiltration(i)
Infiltration(i) = Infiltration(i) * w.FracPerv
Excess(i) = w.FracDCIA * ImpRunoff + w.FracPerv * PervRunoff
TotalExcess = Excess(i) + TotalExcess
Runoff(i) = 60 * Excess(i) * TotalArea * 1.008 / RainTimeStep
TotalInfiltration = TotalInfiltration + Infiltration(i)

LOCATE 2, 2
END SUB

DEFINT A-H, J-Y
FUNCTION GetNode (Message$, FlowToFlag, ActiveFlag)
' FlowToFlag -  Enables checking for other nodes flowing to node and does not list
' ActiveFlag -  Does not list current Active Node

DIM a(MAXNODES)
DIM a$(MAXNODES)

LastActiveNode = ActiveNode

j = 1
FOR i = 1 TO MAXNODES
        IF FlowToFlag = TRUE AND FlowTo(i) = 0 THEN FlowFlag = TRUE
        IF FlowToFlag = FALSE THEN FlowFlag = TRUE
        IF i = ActiveNode AND ActiveFlag = TRUE THEN ActFlag = FALSE ELSE ActFlag = TRUE
        IF NodeExist(i) = TRUE AND FlowFlag = TRUE AND ActFlag = TRUE THEN
                a$(j) = "Node " + STR$(i) + "  " + NodeLabel$(i)
                a(j) = i
                j = j + 1
        END IF
NEXT i

j = j - 1
IF j = 0 THEN GetNode = 0: EXIT FUNCTION
DummyNode = MultAnswer(j, a$(), Message$)
DummyNode = a(DummyNode)
GetNode = DummyNode
colordisplay
END FUNCTION

SUB GetSaveFileName
SaveFileName$ = ""
Direct$ = CURDIR$ + "\"
SaveFileName$ = GetLabel$(Direct$, "Enter a File Name (no Extension)")

IF INSTR(SaveFileName$, " ") OR INSTR(SaveFileName$, ".") THEN
        junk = Alert(4, "Illegal File Name|Do not use spaces or . in filename", 5, 10, 8, 70, "", "", "")
        GetSaveFileName
END IF
END SUB

FUNCTION GetTimeCon (g%, n!, s!, q!, z!, d!, l!)
STATIC y!
ON LOCAL ERROR GOTO tchandler
IF NodeGeometry(ActiveNode) = 2 THEN
        IF y! = 0 THEN y! = 2: incr! = .1
        fy! = 1
        WHILE ABS(fy!) > .001
                dum1! = ((d! + z! * y!) * y! / (d! + 2 * y! * (1 + z! ^ 2) ^ .5)) ^ .6667
                fy! = (d! * y! * (1.486 / n!) * dum1! * s! ^ .5) - q!
                IF SGN(fy!) <> SGN(fy2!) THEN incr! = incr! / 2
                IF fy! > 0 THEN y! = y! - incr! ELSE y! = y! + incr!
                IF incr! = 0 THEN incr! = .1
                fy2! = fy!
        WEND
        v! = q! / ((d! + z! * y!) * y!)
        IF y! > Depth(ActiveNode) THEN Depth(ActiveNode) = y!
ELSE
        IF y! = 0 THEN y! = 1.55
        incr! = .1
        fy! = 1
        WHILE ABS(fy!) > .001
                fy! = (1 / 8) * (y! - SIN(y!)) * d! ^ 2 * (1.486 / n!) * (.25 * (1 - SIN(y!) / y!) * d!) ^ .6667 * s! ^ .5 - NodeInflow(ActiveNode, 1)
                IF SGN(fy!) <> SGN(fy2!) THEN incr! = incr! / 2
                IF fy! > 0 THEN y! = y! - incr! ELSE y! = y! + incr!
                IF incr! = 0 THEN incr! = .1
                fy2! = fy!
        WEND
        IF y! > .1 THEN v! = q! / ((y! - SIN(y!) / 8) * d! ^ 2)
END IF
IF v! > velocity(ActiveNode) THEN velocity(ActiveNode) = v!
IF v! > .01 THEN a = CINT(l! / (v! * 60))
IF v! = 0 THEN a = RainTimeStep
GetTimeCon = a
EXIT FUNCTION
tchandler:
        a$ = "Error !! Parameter Overflow|To Correct, Unlink Node and Relink using another method."
        junk = Alert(4, a$, 9, 10, 14, 70, "", "", "")
        GetTimeCon = -1
        EXIT FUNCTION
END FUNCTION

SUB HyetoPlot (xmax!)
Hyetox1! = 0
FOR i = 1 TO NumRainSteps - 1
        Hyetoy1! = (Excess(i) * 50)
        Hyetox1! = 90 + ((i - 1) * RainTimeStep) / xmax! * 550
        Hyetox2! = 90 + (i * RainTimeStep) / xmax! * 550
        LINE (Hyetox1!, 1)-(Hyetox1!, Hyetoy1!)
        LINE (Hyetox1!, Hyetoy1!)-(Hyetox2!, Hyetoy1!)
        LINE (Hyetox2!, Hyetoy1!)-(Hyetox2!, 1)
NEXT i

LINE (90, 1)-(Hyetox2!, 1)
LINE (90, 12)-(95, 12)
LINE (90, 25)-(95, 25)
LOCATE 2, 4: PRINT ".25 in."
LOCATE 1, 1
PRINT "Excess"

END SUB

SUB LinkNode (p() AS pondtype)
DIM PeakNodeInflow      AS SINGLE
DIM c                   AS SINGLE
DIM Depth               AS SINGLE
DIM Manning             AS SINGLE
DIM SideSlope           AS SINGLE
DIM Diameter            AS SINGLE
DIM Slope               AS SINGLE
DIM Bottom              AS SINGLE
DIM Length              AS SINGLE
DIM a(MAXNODES)
DIM a$(MAXNODES)


ActiveNode = GetNode("Enter Source Node Number", TRUE, FALSE)
IF ActiveNode = 0 THEN EXIT SUB

FlowTo(ActiveNode) = GetNode("Enter Reception Node Number", TRUE, TRUE)
IF FlowTo(ActiveNode) = 0 THEN EXIT SUB

a$(1) = "Circular"
a$(2) = "Trapezoidal"
a$(3) = "No Transport Structure"

IF NodeType(ActiveNode) = 1 OR NodeType(ActiveNode) = 3 OR NodeType(ActiveNode) = 5 THEN
        NodeGeometry(ActiveNode) = GetAnswer(3, a$(), "Choose Transport Geometry")
END IF

IF NodeGeometry(ActiveNode) = 1 OR NodeGeometry(ActiveNode) = 2 THEN
        Slope(ActiveNode) = VAL(GetLabel$("", "Enter The Slope as a fraction"))
        Manning = VAL(GetLabel$(".013", "Enter Manning's Coefficient"))
        Length = VAL(GetLabel$("", "Enter the channel/pipe length in feet"))
        IF Length <= 0 THEN NodeGeometry(ActiveNode) = 3
        IF NodeGeometry(ActiveNode) = 2 THEN SideSlope = VAL(GetLabel$(".25", "Enter side slope vert/horiz"))
END IF


FOR j = 1 TO MAXSTEPS
        IF NodeInflow(ActiveNode, j) > PeakNodeInflow THEN PeakNodeInflow = NodeInflow(ActiveNode, j)
        IF NodeGeometry(ActiveNode) = 3 THEN
                NodeOutflow(ActiveNode, j) = NodeInflow(ActiveNode, j)
                NodeInflow(FlowTo(ActiveNode), j) = NodeOutflow(ActiveNode, j)
        END IF
NEXT j

IF NodeGeometry(ActiveNode) = 3 THEN colordisplay: EXIT SUB
IF NodeType(ActiveNode) = 2 THEN PondRoute ActiveNode, p(): colordisplay: EXIT SUB

IF NodeGeometry(ActiveNode) = 1 THEN
        Diameter = 12 * (PeakNodeInflow * Manning * 2.16 / Slope(ActiveNode) ^ .5) ^ .375
        Diameter = GetDiameter(Diameter, ActiveNode)
        d! = Diameter / 12
END IF

IF NodeGeometry(ActiveNode) = 2 THEN
        z! = 1 / SideSlope
        c! = 2 * ((SQR(1 + z! ^ 2)) - (z!))
        Depth = (PeakNodeInflow * Manning * 1.07 / ((c! + z!) * Slope(ActiveNode) ^ .5)) ^ .375
        Bottom = c! * Depth
        d! = Bottom
        IF d! = 0 THEN d! = 1
        NodeTimeCon = GetTimeCon(2, Manning, Slope(ActiveNode), NodeInflow(ActiveNode, 1), z!, d!, Length)
        IF NodeTimeCon = -1 THEN EXIT SUB
END IF

i = 1
NodeOutflow(ActiveNode, 0) = 0

WHILE Steps < MAXSTEPS
        IF NodeInflow(ActiveNode, i) > .001 THEN
                NodeTimeCon = GetTimeCon(NodeGeometry(ActiveNode), Manning, Slope(ActiveNode), NodeInflow(ActiveNode, i), z!, d!, Length)
                IF NodeTimeCon = -1 THEN EXIT SUB
        ELSE
                NodeTimeCon = 0
        END IF
        KR! = CSNG(RainTimeStep / (2 * NodeTimeCon + RainTimeStep))
        NodeOutflow(ActiveNode, i) = NodeOutflow(ActiveNode, i - 1) + KR! * (NodeInflow(ActiveNode, i - 1) + NodeInflow(ActiveNode, i) - 2 * NodeOutflow(ActiveNode, i - 1))
        IF NodeOutflow(ActiveNode, i) > PeakNodeOutflow THEN PeakNodeOutflow = NodeOutflow(ActiveNode, i): p(ActiveNode).peakstep = i
        IF Offset = 0 THEN Offset = NodeTimeCon / RainTimeStep
        i = i + 1
        Steps = i
        LOCATE 5, 15
        PRINT 2, "Routing in Progress Step:" + STR$(i) + "   "
WEND

IF Offset > 0 THEN Offset = Offset - 1
IF Steps + Offset > MAXSTEPS - 2 THEN Steps = MAXSTEPS - 2 - Offset

FOR i = Offset + Steps TO Offset STEP -1
        NodeOutflow(ActiveNode, i) = NodeOutflow(ActiveNode, i - Offset)
        IF NodeInflow(ActiveNode, i) < .0001 * PeakNodeOutflow THEN NodeInflow(ActiveNode, i) = 0
        IF NodeOutflow(ActiveNode, i) < .0001 * PeakNodeOutflow THEN NodeOutflow(ActiveNode, i) = 0
NEXT i

FOR i = 1 TO Offset
        NodeOutflow(ActiveNode, i) = 0
NEXT i

FOR j = 1 TO MAXSTEPS
        IF NodeOutflow(ActiveNode, j - 1) < .01 AND NodeOutflow(ActiveNode, j) > .01 AND j > p(ActiveNode).peakstep THEN NodeOutflow(ActiveNode, j) = 0
        NodeInflow(FlowTo(ActiveNode), j) = NodeOutflow(ActiveNode, j) + NodeInflow(FlowTo(ActiveNode), j)

NEXT j


Length(ActiveNode) = Length
Diameter(ActiveNode) = d!
Bas(ActiveNode) = Bottom
IF z! <> 0 THEN SideSlope(ActiveNode) = 1 / z!

colordisplay
END SUB

SUB MakeDimRainFall
DIM a$(5)
DIM DimTime             AS SINGLE
DIM RainLastStep        AS SINGLE
DIM TotalPrecip         AS SINGLE
DIM Precip              AS SINGLE
DIM ActualPrecip        AS SINGLE

TotalPrecip = VAL(GetLabel$("", "Input Total Precipitation (in):"))

a$(1) = "Type II"
a$(2) = "Type II FL"
a$(3) = "Type III"

DistType = GetAnswer(3, a$(), "Choose Curve Type")

IF DistType = 0 THEN EXIT SUB

RainLastStep = 0
LOCATE 2, 2
FOR i = 1 TO NumRainSteps
        DimTime = i * RainTimeStep / (RainDuration * 60)
        IF DistType = 1 THEN
                IF DimTime <= 1 THEN Precip = .0178 + 1.893 * DimTime - .9192 * DimTime ^ 2
                IF DimTime = .5 THEN Precip = .663
                IF DimTime < .5 THEN Precip = 22.394 - 101.8 * DimTime + 116.5 * DimTime ^ 2
                IF DimTime < .44 THEN Precip = .4395 * DimTime ^ 1.195
                IF DimTime < .0271 THEN Precip = .226 * DimTime
                
        END IF
        IF DistType = 2 THEN
                IF DimTime <= 1 THEN Precip = -.0508 + 2.0277 * DimTime - .9855 * DimTime ^ 2
                IF DimTime = .5 THEN Precip = .607
                IF DimTime < .5 THEN Precip = .00953 + .0726 * DimTime + .9643 * DimTime ^ 2
                IF DimTime < .0271 THEN Precip = .226 * DimTime
        END IF
        IF DistType = 3 THEN
                IF DimTime <= 1 THEN Precip = 1.003 * DimTime ^ .2657
                IF DimTime < .7 THEN Precip = -3.8077 + 13.7517 * DimTime - 10.009 * DimTime ^ 2
                IF DimTime = .5 THEN Precip = .5
                IF DimTime <= .5 THEN Precip = .01 * EXP(7.212 * DimTime)
                IF DimTime < .0271 THEN Precip = .224 * DimTime
        END IF
        
        Rain(i) = (Precip - RainLastStep) * TotalPrecip
        IF Rain(i) < .01 THEN Rain(i) = .01
        RainLastStep = Precip

NEXT i
ActualPrecip = 0

FOR i = 1 TO NumRainSteps
        Rain(i) = INT(Rain(i) * 100) / 100
NEXT i

FOR i = 1 TO NumRainSteps
        IF Rain(i) > m! THEN m! = Rain(i): mi = i
        ActualPrecip = ActualPrecip + Rain(i)
        IF Rain(i) < 0 THEN Rain(i) = 0
NEXT i
        Rain(mi) = Rain(mi) + INT((TotalPrecip - ActualPrecip + .01) * 100) / 100
END SUB

SUB MakeStageStorDisRel (pond AS pondtype)
DIM a$(5)
DIM SrfArea(9)         AS SINGLE


a$(1) = "60 degree V-Notch Weir"
a$(2) = "90 degree V-Notch Weir"
a$(3) = "Sharp Crested Rectangular Weir"
a$(4) = "Multiple Rectangular Weirs"

WeirType = GetAnswer(4, a$(), "Define Weir Type:")
IF WeirType = 0 THEN EXIT SUB

IF pond.InitStage = 0 THEN pond.InitStage = VAL(GetLabel$("", "Input Invert Stage (ft)"))
pond.StorDisLimit(1) = VAL(GetLabel$("", "What is Storage (acre-feet) at Invert"))

FOR i = 1 TO MAXNODES
        pond.Stage(i) = pond.InitStage
NEXT i

IF pond.NumStorDisRel = 0 THEN pond.NumStorDisRel = VAL(GetLabel$("", "How Many Relationships"))
IF pond.StageInt = 0 THEN pond.StageInt = VAL(GetLabel$("1", "Input Increment of Relationships (ft)"))

WindowOpen 1, 4, 5, 8 + pond.NumStorDisRel, 75, 0, 7, 0, 7, 15, FALSE, FALSE, FALSE, TRUE, 1, ""

windowlocate 1, 4
windowprint -1, "Stage(ft)       Surface Area (acre) "
windowline 2

FOR i = 1 TO pond.NumStorDisRel
        windowlocate 2 + i, 6
        windowprint -1, STR$(pond.InitStage + (i - 1) * pond.StageInt)
        editfieldopen i, "", i + 2, 25, 14, 1, 10, 10
NEXT i

StorFlag = ControlEditWindow(pond.NumStorDisRel, 5 + pond.NumStorDisRel, help$)

FOR i = 1 TO pond.NumStorDisRel
        SrfArea(i) = VAL(EditFieldInquire(i))
NEXT i

FOR i = 2 TO pond.NumStorDisRel
        pond.StorDisLimit(i) = pond.StorDisLimit(i - 1) + (SrfArea(i) + SrfArea(i - 1)) / 2 * pond.StageInt
NEXT i

        SELECT CASE WeirType
                CASE 1
                        DisCoeff! = GetValue!(1.43, "Input Discharge Coefficient")
                        FOR i = 1 TO pond.NumStorDisRel
                                Height! = pond.StageInt * (i - 1)
                                pond.StorDis(i) = DisCoeff! * (Height!) ^ 2.5
                        NEXT i
                CASE 2
                       DisCoeff! = GetValue!(2.5, "Input Discharge Coefficient")
                        FOR i = 1 TO pond.NumStorDisRel
                                Height! = pond.StageInt * (i - 1)
                                pond.StorDis(i) = DisCoeff! * (Height!) ^ 2.5
                        NEXT i
                CASE 3 TO 4
                        Bse! = GetValue(1, "Input Base Length (ft)")
                        DisCoeff! = GetValue!(3.33, "Input Discharge Coefficient")
                        IF WeirType = 4 THEN Weirs = VAL(GetLabel$("", "Number of Weirs")) ELSE Weirs = 1
                        FOR i = 1 TO pond.NumStorDisRel
                                Height! = pond.StageInt * (i - 1)
                                pond.StorDis(i) = (Bse! - .2 * Height!) * Weirs * DisCoeff! * (Height!) ^ 1.5
                        NEXT i
                END SELECT
WindowClose 1


END SUB

SUB PlotAxis (xmax!, ymax!)

DIM xlabel(11) AS SINGLE
DIM ylabel(10) AS SINGLE

Message$ = "Enter Label for Plot"
PlotLabel$ = GetLabel$("", Message$)

CLS
SCREEN 2, 0

' Draw Axes

LINE (90, 160)-(90, 0)

LINE (90, 160)-(640, 160)

' Draw Tick Marks on axes

xlabel(1) = 0
FOR i = 1 TO 11
        LINE (85, i * 12 + 28)-(95, i * 12 + 28)
NEXT i

FOR i = 1 TO 10
        LINE (90 + i * 55, 155)-(90 + i * 55, 165)
        xlabel(i + 1) = (xmax! / 600 + xlabel(i))
        ylabel(i) = (ymax! * 11 / 12 * i / 10)
NEXT i

LOCATE 3, 48: PRINT "Peak Flow:"; : PRINT USING "###.# cfs"; ymax!
LOCATE 4, 48: PRINT PlotLabel$

cfs$ = "Flow  cfs"
FOR i = 1 TO 9
        LOCATE i + 7, 1: PRINT MID$(cfs$, i, 1)
NEXT i
LOCATE 23, 30
PRINT "Time (hrs)"

kgk = 0
FOR i = 1 TO 10
        LOCATE 22, 10 + kgk
        PRINT USING "##.#"; xlabel(i)
        kgk = kgk + 7
NEXT i

LOCATE 20, 4: PRINT USING "###.#"; 0
LOCATE 17, 4: PRINT USING "###.#"; ylabel(2)
LOCATE 14, 4: PRINT USING "###.#"; ylabel(4)
LOCATE 11, 4: PRINT USING "###.#"; ylabel(6)
LOCATE 8, 4: PRINT USING "###.#"; ylabel(8)
LOCATE 5, 4: PRINT USING "###.#"; ylabel(10)

END SUB

SUB PlotHydrograph
DIM a$(2)

a$(1) = "W/ Rainfall Excess"
a$(2) = "W/O Rainfall Excess"
Hyeto = GetAnswer(2, a$(), "Specify plotting choice")

PlotAxis CSNG(HydroSteps * RainTimeStep), PeakOutflow

PlotSeries HydroSteps, HydroSteps, Outflow(), PeakOutflow

IF Hyeto = 1 THEN HyetoPlot CSNG(HydroSteps * RainTimeStep)

DO
        c$ = INKEY$
        MousePoll R, c, lb, rb
LOOP UNTIL c$ <> "" OR lb OR rb

SCREEN 0, 0, 0

MenuShow
MouseShow
colordisplay

END SUB

SUB PlotNode
DIM PlotN(MAXNODES)
DIM InSteps(MAXNODES)
DIM OutSteps(MAXNODES)
DIM flow!(MAXSTEPS)
DIM flow2!(MAXSTEPS)
DIM a$(MAXNODES)
DIM b$(2)
DIM a(MAXNODES)

b$(1) = "No"
b$(2) = "Yes"

ExitFlag = FALSE

WHILE NOT ExitFlag
        dummy = GetNode("Choose Node To Plot", FALSE, FALSE)
        IF dummy <> 0 THEN PlotN(dummy) = TRUE ELSE EXIT SUB
        junk = GetAnswer(2, b$(), "Plot another on same plot?")
        IF junk = 1 THEN ExitFlag = TRUE
     
WEND

Stepsmax = VAL(GetLabel$("200", "Choose Number of steps to plot < 220"))
IF Stepsmax = 0 OR Stepsmax > MAXSTEPS - 1 THEN Stepsmax = MAXSTEPS - 1

FOR i = 1 TO MAXNODES
        IF PlotN(i) = TRUE THEN
                FOR j = 1 TO Stepsmax
                        IF NodeInflow(i, j) > NodalMax! THEN NodalMax! = NodeInflow(i, j)
                        IF NodeOutflow(i, j) > NodalMax! THEN NodalMax! = NodeOutflow(i, j)
                NEXT j
        END IF
NEXT i


PlotAxis CSNG(RainTimeStep * Stepsmax), NodalMax!

FOR i = 1 TO MAXNODES
        IF PlotN(i) = TRUE THEN
                FOR j = 1 TO Stepsmax
                        flow!(j) = NodeInflow(i, j)
                NEXT j
                FOR j = 1 TO Stepsmax
                        flow2!(j) = NodeOutflow(i, j)
                NEXT j

        PlotSeries Stepsmax, Stepsmax, flow!(), NodalMax!
        PlotSeries Stepsmax, Stepsmax, flow2!(), NodalMax!
        END IF
NEXT i

DO
        c$ = INKEY$
        MousePoll R, c, lb, rb
LOOP UNTIL c$ <> "" OR lb OR rb

SCREEN 0, 0, 0

MenuShow
MouseShow
colordisplay

END SUB

'
SUB PlotSeries (Steps, Stepsmax, Valy!(), ymax!)

DIM valx!(1 TO Steps + 1)

xmax! = CSNG(Stepsmax * RainTimeStep)
FOR j = 1 TO Steps
        valx!(j) = CSNG(j * RainTimeStep)
        IF valx!(j) > xmax! THEN xmax! = valx!(j)
        IF Valy!(j) > ymax! THEN ymax! = Valy!(j)
NEXT j
' Draw Hydrograph Plot

FOR i = 1 TO Steps
x1! = 90 + valx!(i) / xmax! * 550
y1! = 160 - Valy!(i) / ymax! * 120
x2! = 90 + valx!(i + 1) / xmax! * 550
y2! = 160 - Valy!(i + 1) / ymax! * 120

IF x2! >= x1! THEN LINE (x1!, y1!)-(x2!, y2!)
NEXT i


END SUB

SUB PondRoute (ActiveNode, p() AS pondtype)
DIM a$(2)
DIM BackTime!(10)
DIM BackStages!(10)

a$(1) = "No": a$(2) = "Yes"
BackWaterFlag = GetAnswer(2, a$(), "Do You Have a backwater relationship?")

IF BackWaterFlag = 2 THEN
        NumBackStages = VAL(GetLabel$("", "How Many? ( <10)"))
        FOR j = 1 TO NumBackStages
                BackTime!(j) = VAL(GetLabel$("", "Input Backwater Time " + STR$(j) + " (hr)"))
                BackStages!(j) = VAL(GetLabel$("", "Input Backwater Stage " + STR$(j) + " (ft)"))
        NEXT j
END IF

volume! = p(ActiveNode).OrigVolume * 43560

i = 1
WHILE i < MAXSTEPS - 1
        k = p(ActiveNode).NumStorDisRel - 1
        DO WHILE k >= 0
                IF volume! >= p(ActiveNode).StorDisLimit(k) * 43560 THEN
                        f! = (volume! - (p(ActiveNode).StorDisLimit(k) * 43560!)) / ((p(ActiveNode).StorDisLimit(k + 1) - p(ActiveNode).StorDisLimit(k)) * 43560!)
                        StartStage! = (p(ActiveNode).InitStage + p(ActiveNode).StageInt * (k - 1))
                        NextStage! = (p(ActiveNode).InitStage + p(ActiveNode).StageInt * k)
                        Stage! = StartStage! + p(ActiveNode).StageInt * f!
                        flow! = p(ActiveNode).StorDis(k) + (Stage! - StartStage!) / (NextStage! - StartStage!) * ((p(ActiveNode).StorDis(k + 1) - p(ActiveNode).StorDis(k)))
                        EXIT DO
                END IF
        k = k - 1
        LOOP

        IF BackWaterFlag = 2 THEN
                BackStage! = GetBackStage!(i, NumBackStages, BackTime!(), BackStages!())
                unflow! = 0
                IF BackStage! > Stage! THEN
                        flow! = 0
                ELSEIF BackStage! <> 0 AND Stage! > BackStage! AND BackStage! > p(ActiveNode).InitStage THEN
                        unflow! = p(ActiveNode).StorDis(k) + (BackStage! - StartStage!) / (NextStage! - StartStage!) * ((p(ActiveNode).StorDis(k + 1) - p(ActiveNode).StorDis(k)))
                        IF unflow! < 0 THEN unflow! = 0
                        flow! = flow! - unflow!
                END IF
                IF flow! < 0 THEN flow! = 0
        END IF

        IF flow! < 0 THEN flow! = 0
                NodeOutflow(ActiveNode, i) = flow!
                volume! = volume! + (NodeInflow(ActiveNode, i) - NodeOutflow(ActiveNode, i)) * RainTimeStep * 60
                IF volume! > p(ActiveNode).maxvolume THEN p(ActiveNode).maxvolume = volume!
                p(ActiveNode).Stage(i) = Stage!
                IF Stage! > p(ActiveNode).MaxStage THEN p(ActiveNode).MaxStage = Stage!
                i = i + 1
WEND

FOR j = 1 TO MAXSTEPS - 1
        NodeInflow(FlowTo(ActiveNode), j) = NodeOutflow(ActiveNode, j) + NodeInflow(FlowTo(ActiveNode), j)
        IF NodeOutflow(ActiveNode, j) > peakpondflow! THEN peakpondflow! = NodeOutflow(ActiveNode, j): p(ActiveNode).peakstep = j
NEXT j

FOR i = 1 TO MAXSTEPS
        IF p(ActiveNode).Stage(i) = 0 THEN p(ActiveNode).Stage(i) = p(ActiveNode).InitStage
NEXT i

END SUB

SUB PrintHydrograph

a = routeprint
IF a = FALSE THEN EXIT SUB
label$ = GetLabel$("", "Enter a Label for Printing")
IF a = 3 THEN CLS
page = 1
CumRain! = 0
WHILE i < HydroSteps
        PRINT #1, TAB(10); STRING$(50, "-"); "SMADA  PAGE"; page
        PRINT #1, TAB(8); label$; "    Hydrograph Type :"; Hydrotype$
        PRINT #1, bar$
        PRINT #1, "       Time      Rain   Cumulative  Infiltration Instantaneous  Outflow "
        PRINT #1, "       (hr)      (in)     (in)          (in)         (cfs)       (cfs)"
        PRINT #1, bar$
        FOR i = i1 TO i1 + 49
                CumRain! = CumRain! + Rain(i)
                IF a = 3 AND i / 15! = INT(i / 15) THEN WHILE INKEY$ = "": WEND
                PRINT #1, USING "       ##.##"; i * RainTimeStep / 60;
                PRINT #1, USING "       ##.##"; Rain(i);
                PRINT #1, USING "      ###.##"; CumRain!;
                PRINT #1, USING "       ##.##"; Infiltration(i);
                PRINT #1, USING "     ####.##"; Runoff(i);
                PRINT #1, USING "      ###.##"; Outflow(i)
        IF i = HydroSteps THEN i = i1 + 51
        NEXT i
        i1 = i
        IF i < HydroSteps THEN
                PRINT #1, bar$;
                PRINT #1, CHR$(12)
        ELSE
                PRINT #1, bar$
                PRINT #1, USING "        Total      ##.##"; CumRain!;
                PRINT #1, " in";
                PRINT #1, TAB(41);
                PRINT #1, USING "########"; TotalRunoff * RainTimeStep * 60;
                PRINT #1, USING " cf ######## cf"; TotalOutflow * RainTimeStep * 60
                PRINT #1, USING "        Rational Coefficient  = #.###"; RatCoeff;
                PRINT #1, TAB(42);
                PRINT #1, USING "Peak Flow =  ###.## cfs"; PeakOutflow;
                PRINT #1, TAB(10);
                IF Hydrotype$ = "Santa Barbara Method" THEN
                        PRINT #1, USING "Routing Coefficient = #.###"; KR!
                ELSE
                        PRINT #1, USING "Attenuation Factor = ###"; PeakAttFact
                END IF
                PRINT #1, TAB(10); "Watershed File: "; WaterFile$; "    Rainfall File: "; RainFile$
                PRINT #1, bar$
        END IF
page = page + 1
WEND
IF a = 3 THEN WHILE INKEY$ = "": WEND
PRINT #1, CHR$(12)
CLOSE
colordisplay
END SUB

SUB PrintNode (p() AS pondtype)

DIM a$(MAXNODES)

a = routeprint
IF a = FALSE THEN EXIT SUB
DummyNode = GetNode("Choose Node to Print", FALSE, FALSE)
IF DummyNode = 0 THEN EXIT SUB ELSE ActiveNode = DummyNode
IF a = 3 THEN CLS
i = 1
PRINT #1, TAB(10); "Node : "; ActiveNode; " - "; NodeLabel$(ActiveNode); " Flowing To "; NodeLabel$(FlowTo(ActiveNode));
PRINT #1, TAB(60); " SMADA PAGE 1"
SELECT CASE NodeType(ActiveNode)
        CASE 1: a$ = " Watershed Node"
        CASE 3: a$ = " Junction Node"
        CASE 5: a$ = " Batch Node"
END SELECT
        IF NodeGeometry(ActiveNode) = 1 THEN
                b$ = " Circular Outlet "
        ELSEIF NodeGeometry(ActiveNode) = 2 THEN
                b$ = " Trapezoidal Outlet "
        END IF
IF NodeType(ActiveNode) = 2 THEN
        a$ = " Pond Node:"
        b$ = "Output by Storage Discharge Relationship"
END IF

PRINT #1, TAB(10); a$; b$
IF (NodeType(ActiveNode) = 5 OR NodeType(ActiveNode) = 1 OR NodeType(ActiveNode) = 3) AND NodeGeometry(ActiveNode) <> 3 THEN
        PRINT #1, TAB(10);
        PRINT #1, USING " Length = ###.# ft"; Length(ActiveNode);
        PRINT #1, USING "  Longitudinal Slope = #.###"; Slope(ActiveNode)
        PRINT #1, TAB(10);
        PRINT #1, USING "  Velocity (at full flow conditions) = ###.## fps"; velocity(ActiveNode)
        PRINT #1, TAB(10);
        IF NodeGeometry(ActiveNode) = 1 THEN
                PRINT #1, USING "Diameter = ##.## ft"; Diameter(ActiveNode);
        ELSEIF NodeGeometry(ActiveNode) = 2 THEN
                PRINT #1, USING "Base = ##.## ft"; Diameter(ActiveNode);
                PRINT #1, USING "  Side v/h = #.##"; SideSlope(ActiveNode);
                PRINT #1, USING "  Max Depth = ##.## ft"; Depth(ActiveNode)
        END IF

ELSEIF NodeType(ActiveNode) = 2 THEN
        PRINT #1, TAB(10);
        PRINT #1, USING "Maximum Volume = ####.## acre-feet   Maximum Stage = ####.## ft"; p(ActiveNode).maxvolume / 43560; p(ActiveNode).MaxStage
        IF p(ActiveNode).NumStorDisRel <> 0 THEN
                PRINT #1,
                PRINT #1, TAB(10); " Stage - Storage - Discharge Relationship "
                PRINT #1, TAB(10); "Stage (ft)      Storage (Acre-ft)    Discharge (cfs)"
                FOR i = 1 TO p(ActiveNode).NumStorDisRel
                        PRINT #1, TAB(10);
                        PRINT #1, USING "  ###.#          ####.##          ####.##"; p(ActiveNode).InitStage + (i - 1) * p(ActiveNode).StageInt; p(ActiveNode).StorDisLimit(i); p(ActiveNode).StorDis(i)
                NEXT i
                PRINT #1,
        END IF
END IF
IF a = 3 THEN WHILE INKEY$ = "": WEND
PRINT #1, bar$
ExitFlag = FALSE
IF p(ActiveNode).NumStorDisRel <> 0 THEN
        page = 2: PRINT #1, CHR$(12)
ELSE
        page = 1
END IF
i = 1
WHILE NOT ExitFlag
        ExitFlag2 = FALSE
        j = 1
        
        IF page <> 1 THEN
                PRINT #1, TAB(10); "Node : "; ActiveNode; " - "; NodeLabel$(ActiveNode);
                PRINT #1, TAB(61); "SMADA PAGE "; page: PRINT #1,
        END IF
        PRINT #1, TAB(10); "Time        Inflow (cfs)       Outflow(cfs)";
        IF NodeType(ActiveNode) = 2 THEN PRINT #1, "    Stage (ft)" ELSE PRINT #1,
        PRINT #1, bar$
        WHILE NOT ExitFlag2
                IF a = 3 THEN WHILE INKEY$ = "": WEND
                PRINT #1, USING "        ###.##"; i * RainTimeStep / 60;
                PRINT #1, USING "       ###.##"; NodeInflow(ActiveNode, i);
                PRINT #1, USING "              ###.##"; NodeOutflow(ActiveNode, i);
                IF NodeType(ActiveNode) = 2 THEN PRINT #1, USING "          ###.##"; p(ActiveNode).Stage(i) ELSE PRINT #1,
                ExitFlag2 = ((j = 50) OR (i = MAXSTEPS - 1))
                i = i + 1
                j = j + 1
        WEND
        ExitFlag = (i = MAXSTEPS - 1) OR (i > p(ActiveNode).peakstep AND NodeOutflow(ActiveNode, i) < .001)
        PRINT #1, bar$;
        PRINT #1, CHR$(12)
        page = page + 1
WEND
CLOSE #1
colordisplay

END SUB

DEFINT Z
SUB PrintOrderForm
a = routeprint
IF a = 0 THEN EXIT SUB
IF a = 3 THEN CLS
OPEN "order.frm" FOR INPUT AS #2
        i = 1
        WHILE NOT EOF(2)
                LINE INPUT #2, a$
                PRINT #1, a$
                i = i + 1
                IF i / 20! = INT(i / 20) AND a = 3 THEN WHILE INKEY$ = "": WEND
        WEND

        IF a = 3 THEN WHILE INKEY$ = "": WEND
        IF a = 1 THEN PRINT #1, CHR$(12)
CLOSE #1
CLOSE #2
END SUB

DEFSNG Z
SUB PrintWatershed (w AS watershedtype)
a = routeprint
IF a = FALSE THEN EXIT SUB
label$ = GetLabel$("", "Enter Print Label for Watershed")
IF a = 3 THEN CLS
PRINT #1, TAB(15); "SMADA 4.0   Watershed Information for: "; label$
PRINT #1, bar$
PRINT #1, USING "        Total Watershed area ###.# acres"; TotalArea
PRINT #1, USING "        Total Impervious area ###.# acres"; w.ImpArea
PRINT #1, USING "        Total Percent impervious area which is directly connected: ###.# %"; w.DCIA
PRINT #1, USING "        Time of concentration: ### minutes"; w.TimeCon
PRINT #1, "        Additional Abstraction for different area types"
PRINT #1, USING "        Pervious #.## inches     Impervious #.## inches"; w.AddAbsImp; w.AddAbsPerv
PRINT #1, bar$
IF w.infmeth = 2 THEN
        PRINT #1, "        Infiltration Calculated by Horton Method"
        PRINT #1, USING "        Initial Infiltration Rate ##.## in/hr"; w.HIO
        PRINT #1, USING "        Limiting Infiltration Rate ##.## in/hr"; w.HIC
        PRINT #1, USING "        Depletion Coefficient ##.## hr^-1"; w.HK
ELSE
        PRINT #1, "        Infiltration Calculated by SCS Curve Number Method"
        PRINT #1, USING "        Pervious Curve Number: ##"; w.CN
        PRINT #1, USING "        Initial Abstraction Factor: #.##"; w.InitAbs
END IF
PRINT #1, bar$
PRINT #1, CHR$(12)
CLOSE
IF a = 3 THEN WHILE INKEY$ = "": WEND
colordisplay
END SUB

SUB RainFilePrint

TotalRain = 0
IF RainTimeStep = 0 THEN
        a$ = "No Rainfall File Active in Memory"
        junk = Alert(4, a$, 9, 10, 14, 70, "", "", "")
        EXIT SUB
END IF
a = routeprint
IF a = FALSE THEN EXIT SUB
PRINT #1, TAB(12); "Time          Rainfall"
PRINT #1, TAB(12); "(min)           (in)"
PRINT #1, bar$
FOR i = 1 TO NumRainSteps
        IF a = 3 THEN WHILE INKEY$ = "": WEND
        TotalRain = TotalRain + Rain(i)
        PRINT #1, TAB(12);
        PRINT #1, USING "#####"; i * RainTimeStep;
        PRINT #1, USING "         ####.##"; Rain(i)
NEXT i
PRINT #1, bar$
PRINT #1, TAB(12); "Total Rain = ";
PRINT #1, USING "###.##"; TotalRain
CLOSE
colordisplay
END SUB

SUB RetrieveHydroFile
RFile$ = FileRetrieve$("*.hyd")
IF RFile$ <> "" THEN
        NumRainSteps = 0
        TotalRain = 0
        TotalInfiltration = 0
        TotalRunoff = 0
        TotalOutflow = 0
        PeakOutflow = 0
        FOR i = 1 TO MAXSTEPS
                Rain(i) = 0
                Infiltration(i) = 0
                Runoff(i) = 0
                Outflow(i) = 0
        NEXT i

        OPEN RFile$ FOR INPUT AS #1
        INPUT #1, Hydrotype$
        IF Hydrotype$ = "Santa Barbara Method" THEN INPUT #1, KR! ELSE INPUT #1, PeakAttFact
        INPUT #1, HydroSteps, RainTimeStep, TotalArea
        FOR i = 1 TO HydroSteps
                INPUT #1, Rain(i), Infiltration(i), Runoff(i), Outflow(i)
                IF Rain(i) <> 0 THEN NumRainSteps = NumRainSteps + 1
                TotalRain = TotalRain + Rain(i)
                TotalInfiltration = TotalInfiltration + Infiltration(i)
                TotalRunoff = TotalRunoff + Runoff(i)
                TotalOutflow = TotalOutflow + Outflow(i)
                IF Outflow(i) > PeakOutflow THEN PeakOutflow = Outflow(i)
        NEXT i
TotalExcess = TotalRunoff * 60 * RainTimeStep / (TotalArea * 3630)
RatCoeff = TotalExcess / TotalRain
RatCoeff = FIX(RatCoeff * 1000) / 1000
TotalRain = FIX(TotalRain * 100) / 100
HydroFile$ = RFile$: RainFile$ = RFile$: WaterFile$ = RFile$
END IF
CLOSE #1
colordisplay
END SUB

DEFINT Z
SUB RetrieveRainFile
RFile$ = FileRetrieve$("*.rnf")
RainFile$ = RFile$
IF RFile$ <> "" THEN
    OPEN RFile$ FOR INPUT AS #1
    INPUT #1, NumRainSteps, RainDuration, RainTimeStep
    FOR i = 1 TO NumRainSteps
    INPUT #1, Rain(i)
    NEXT i
    CLOSE #1
END IF
END SUB

SUB RetrieveShedFile (w AS watershedtype)
RFile$ = FileRetrieve$("*.shd")
WaterFile$ = RFile$
IF RFile$ <> "" THEN

    OPEN RFile$ FOR INPUT AS #1
    INPUT #1, TotalArea
    INPUT #1, w.TimeCon
    INPUT #1, w.ImpArea
    INPUT #1, w.DCIA
    INPUT #1, w.AddAbsImp
    INPUT #1, w.AddAbsPerv
    INPUT #1, w.MaxInf
    INPUT #1, w.infmeth
    SELECT CASE w.infmeth
    CASE 2
        INPUT #1, w.HIC
        INPUT #1, w.HIO
        INPUT #1, w.HK
    CASE 3
        INPUT #1, w.CN
        INPUT #1, w.InitAbs
    END SELECT
ELSE
        EXIT SUB
END IF

    CLOSE #1
ShedCalc w

END SUB

DEFSNG Z
SUB RetrieveStorDisRel (pond AS pondtype)
RFile$ = FileRetrieve$("*.pnd")
IF RFile$ <> "" THEN
    OPEN RFile$ FOR INPUT AS #1
    INPUT #1, pond.NumStorDisRel, pond.InitStage, pond.StageInt
    FOR i = 1 TO pond.NumStorDisRel
        INPUT #1, pond.StorDisLimit(i), pond.StorDis(i)
    NEXT i
    CLOSE #1
END IF
END SUB

FUNCTION routeprint
DIM a$(3)
a$(1) = "Printer"
a$(2) = "Disk"
a$(3) = "Screen"
route = GetAnswer(3, a$(), "Choose Print Destination")

SELECT CASE route
        CASE 0: routeprint = FALSE: EXIT FUNCTION
        CASE 1:  OPEN "lpt1:" FOR OUTPUT AS #1
        CASE 2
                GetSaveFileName
                IF SaveFileName$ <> "" THEN
                        OPEN SaveFileName$ + ".prn" FOR OUTPUT AS #1
                END IF
        CASE 3: OPEN "scrn:" FOR OUTPUT AS #1

END SELECT

routeprint = route
END FUNCTION

DEFSNG A-H, J-Y
SUB SantaBarbara (w AS watershedtype)

DIM a$(100)

PeakOutflow = 0
TotalOutflow = 0
Runoff(1) = 0
Outflow(1) = 0
ExitFlag = FALSE
Hydrotype$ = "Santa Barbara Method"

IF w.TimeCon = 0 THEN EXIT SUB
IF RainTimeStep = 0 THEN EXIT SUB


i = 1
KR! = RainTimeStep / (2 * w.TimeCon + RainTimeStep)

WHILE NOT ExitFlag

        GetInfiltration i, w

        IF i >= 2 THEN
                Outflow(i) = Outflow(i - 1) + KR! * (Runoff(i - 1) + Runoff(i) - 2 * Outflow(i - 1))
        ELSE
                Outflow(i) = KR! * Runoff(i)
        END IF
        IF Outflow(i) > PeakOutflow THEN PeakOutflow = Outflow(i)

        IF i >= 5 THEN
                IF (Outflow(i) < .005 * PeakOutflow) OR (i = MAXSTEPS - 1) THEN
                        HydroSteps = i
                        ExitFlag = TRUE
                END IF
        END IF

        i = i + 1

WEND

END SUB

DEFINT A-H, J-Y
SUB SaveBatchData
ActiveNode = GetNode("Which Node to Output", FALSE, FALSE)
IF ActiveNode = 0 THEN EXIT SUB
GetSaveFileName
IF SaveFileName$ = "" THEN EXIT SUB
BatFile$ = SaveFileName$ + ".dat"
    OPEN BatFile$ FOR OUTPUT AS #1
    FOR i = 1 TO MAXSTEPS
        PRINT #1, NodeOutflow(ActiveNode, i)
    NEXT i
    CLOSE #1

END SUB

SUB SaveHydroFile
IF HydroSteps = 0 THEN EXIT SUB
GetSaveFileName
IF SaveFileName$ = "" THEN EXIT SUB
HydroFile$ = SaveFileName$ + ".hyd"
OPEN HydroFile$ FOR OUTPUT AS #1
PRINT #1, Hydrotype$
IF Hydrotype$ = "Santa Barbara Method" THEN PRINT #1, KR! ELSE PRINT #1, PeakAttFact
PRINT #1, HydroSteps, RainTimeStep, TotalArea
FOR i = 1 TO HydroSteps
        PRINT #1, Rain(i), Infiltration(i), Runoff(i), Outflow(i)
NEXT i
CLOSE #1
END SUB

DEFINT Z
SUB SaveRainFile
IF NumRainSteps = 0 THEN EXIT SUB
GetSaveFileName
IF SaveFileName$ = "" THEN EXIT SUB
RainFile$ = SaveFileName$ + ".rnf"
    OPEN RainFile$ FOR OUTPUT AS #1
    PRINT #1, NumRainSteps, RainDuration, RainTimeStep
    FOR i = 1 TO NumRainSteps
        PRINT #1, Rain(i)
    NEXT i
    CLOSE #1
  
END SUB

SUB SaveShedFile (w AS watershedtype)
IF w.TimeCon = 0 THEN EXIT SUB
GetSaveFileName
IF SaveFileName$ = "" THEN EXIT SUB

    ShedFile$ = SaveFileName$ + ".shd"

    OPEN ShedFile$ FOR OUTPUT AS #1
    PRINT #1, TotalArea
    PRINT #1, w.TimeCon
    PRINT #1, w.ImpArea
    PRINT #1, w.DCIA
    PRINT #1, w.AddAbsImp
    PRINT #1, w.AddAbsPerv
    PRINT #1, w.MaxInf
    PRINT #1, w.infmeth
    SELECT CASE w.infmeth
    CASE 2
        PRINT #1, w.HIC
        PRINT #1, w.HIO
        PRINT #1, w.HK
    CASE 3
        PRINT #1, w.CN
        PRINT #1, w.InitAbs
    END SELECT
    CLOSE #1

    WindowClose 1

END SUB

DEFSNG Z
SUB SaveStorDisRel (pond AS pondtype)

GetSaveFileName
IF SaveFileName$ = "" THEN EXIT SUB
RFile$ = SaveFileName$ + ".pnd"
    OPEN RFile$ FOR OUTPUT AS #1
    PRINT #1, pond.NumStorDisRel, pond.InitStage, pond.StageInt
    FOR i = 1 TO pond.NumStorDisRel
        PRINT #1, pond.StorDisLimit(i), pond.StorDis(i)
    NEXT i
CLOSE #1

END SUB

DEFSNG A-H, M-Y
SUB SCSMethod (w AS watershedtype)
STATIC UnitHydro()      AS SINGLE
DIM Response!(MAXSTEPS)
DIM currEditField       AS INTEGER
DIM currButton          AS INTEGER
REDIM UnitHydro(MAXSTEPS)    AS SINGLE
DIM Steptime                    AS SINGLE

Hydrotype$ = "Discrete Unit Convolution"
UniformFlag = FALSE
IF PeakAttFact = -1 THEN UniformFlag = TRUE

IF w.TimeCon = 0 THEN EXIT SUB
IF RainTimeStep = 0 THEN EXIT SUB

FOR j = 1 TO HydroSteps
        Outflow(j) = 0
NEXT j

TotalOutflow = 0
PeakOutflow = 0
IF PeakAttFact = 0 THEN

REDIM a$(17)

        a$(1) = "You must choose a value for the Peak Attenuation Factor."
        a$(2) = "   RL = Recession Limb Time    TP = Time to Peak"
        a$(3) = ""
        a$(4) = "645: Rational Hydrograph , Steep Terrain"
        a$(5) = "Note that Rain Duration must = Time of Concentration"
        a$(6) = "580: RL = 1.25 * TP"
        a$(7) = "484: RL = 1.67 * TP, Typical SCS Curve"
        a$(8) = "400: RL = 2.25 * TP"
        a$(9) = "300: RL = 3.33 * TP, Minimum Slope ,Urban"
        a$(10) = "200: RL = 5.50 * TP"
        a$(11) = "100: RL = 12.0 * TP, Rural, Flat Terrain"
         a$(12) = ""
         a$(13) = "50 < K < 646"

WindowOpen 1, 3, 5, 19, 75, 0, 7, 0, 7, 15, FALSE, FALSE, FALSE, TRUE, 1, "Attenuation"

FOR i = 1 TO 13
        windowlocate i, 5
        windowprint 1, a$(i)
NEXT i

editfieldopen 1, "", 17, 20, 14, 1, 5, 5

ButtonOpen 1, 1, "Cancel", 17, 40, 19, 50, 1

ExitFlag = FALSE
currButton = 0
currEditField = 1

WHILE NOT ExitFlag
        WindowDo currButton, currEditField
        SELECT CASE Dialog(0)
        CASE 1:
                WindowClose 1
                EXIT SUB
        CASE 2: currEditField = 1
        CASE 6
                IF ButtonInquire(1) = 2 THEN
                        WindowClose 1
                        EXIT SUB
                END IF
                IF EditFieldInquire(1) <> "" THEN ExitFlag = TRUE
        CASE 7
                ButtonToggle 1
                IF ButtonInquire(1) = 2 THEN currEditField = 0 ELSE currEditField = 1
        CASE 9
                WindowClose 1
                EXIT SUB
        END SELECT
WEND
PeakAttFact = VAL(EditFieldInquire(1))
WindowClose 1
IF PeakAttFact < 50 OR PeakAttFact > 1000 THEN
        a$ = "Illegal Value of Attenuation"
        junk = Alert(4, a$, 9, 10, 12, 70, "", "", "")
        EXIT SUB
END IF


ELSE
Hydrotype$ = "Rational Method"
END IF
IF UniformFlag = TRUE THEN EXIT SUB

' ===========  Generate Unit Hydrograph ===============

FallLimbFact = (1291 / PeakAttFact) - 1


IF w.TimeCon < RainTimeStep THEN
        UnitTimeCon = RainTimeStep
ELSE
        UnitTimeCon = w.TimeCon
END IF

TimeFall = FallLimbFact * w.TimeCon
TotalUnitTime = UnitTimeCon + TimeFall
TotalUnitTime = INT(TotalUnitTime / RainTimeStep + .5) * RainTimeStep
JSteps = INT(TotalUnitTime / RainTimeStep + .5)

IF w.SmoothFlag = FALSE THEN
        FOR j = 1 TO JSteps
                Steptime = j * RainTimeStep
                IF Steptime <= UnitTimeCon THEN
                        UnitHydro(j) = Steptime / UnitTimeCon
                        tp = j
                ELSE
                        UnitHydro(j) = 1 + UnitTimeCon / TimeFall - Steptime / TimeFall
                END IF
        NEXT j
END IF

IF w.SmoothFlag = TRUE THEN
        FOR j = 1 TO JSteps
                Steptime = j * RainTimeStep
                tr! = Steptime / UnitTimeCon
                IF Steptime <= UnitTimeCon THEN
                        UnitHydro(j) = .05903 - .7650173 * tr! + 4.415467 * tr! ^ 2 - 2.708 * tr! ^ 3
                        tp = j
                ELSE
                        IF PeakAttFact = 484 THEN a! = 2.445061: b! = -1.763459: c! = .425012: c! = .425012: d! = -.034003
                        IF PeakAttFact = 256 THEN a! = 1.883346: b! = -.9724275: c! = .1670826: d! = -.009394733#
                        UnitHydro(j) = a! + b! * tr! + c! * tr! ^ 2 + d! * tr! ^ 3
                END IF
        IF UnitHydro(j) < 0 THEN UnitHydro(j) = 0
        NEXT j
END IF

TotalHydro = 0
FOR j = 1 TO JSteps
        TotalHydro = TotalHydro + UnitHydro(j)
NEXT j


FOR j = 1 TO JSteps
        UnitHydro(j) = UnitHydro(j) / TotalHydro
        IF UnitHydro(j) > p! THEN p! = UnitHydro(j)

NEXT j

' ==========   Generate Hydrograph using unit Hydrograph ===============

i = 1
ExitFlag = FALSE
WHILE NOT ExitFlag
        GetInfiltration i, w
        FOR j = 0 TO i - 1
                Outflow(i) = Outflow(i) + Runoff(i - j) * UnitHydro(j + 1)
        NEXT j

        IF Outflow(i) > PeakOutflow THEN PeakOutflow = Outflow(i)
        
        IF i = MAXSTEPS THEN ExitFlag = TRUE
        IF i > 5 AND Outflow(i) < .005 * PeakOutflow THEN ExitFlag = TRUE
        
        i = i + 1
WEND
HydroSteps = i
IF HydroSteps > MAXSTEPS THEN HydroSteps = MAXSTEPS - 1
END SUB

DEFSNG I-L
SUB SetupDesktop STATIC

    MouseHide

    WIDTH , 25
    COLOR 15, 1      'Color
    CLS

    FOR a = 2 TO 80 STEP 4
        FOR b = 2 TO 25 STEP 2
            LOCATE b, a
            PRINT CHR$(250);
        NEXT b
    NEXT a
    LOCATE 25, 1
    COLOR 0, 7: PRINT STRING$(80, " ");
    LOCATE 25, 1
    IF HydroFile$ <> "" THEN PRINT "Current Hydrograph :"; HydroFile$; "  ";
    IF ActiveNode <> 0 THEN PRINT "Current Active Node: "; NodeLabel$(ActiveNode);
    MouseShow
END SUB

DEFINT A-Z
SUB SetupMenu

    MenuSet 1, 0, 1, "Utilities", 1
    MenuSet 1, 1, 1, "Use HELP Utility           F1", 5
    MenuSet 1, 2, 1, "Change Current Directory   F2", 1
    MenuSet 1, 3, 1, "Send Form Feed to Printer  F3", 6
    MenuSet 1, 4, 1, "-", 1
    MenuSet 1, 5, 1, "Exit                   Ctrl-X", 2
                          
    MenuSet 2, 0, 1, "Watershed", 1
    MenuSet 2, 1, 1, "Create Watershed File", 1
    MenuSet 2, 2, 1, "Edit/View Watershed File", 1
    MenuSet 2, 3, 1, "Retrieve Watershed File", 1
    MenuSet 2, 4, 1, "Save Watershed File", 1
    MenuSet 2, 5, 1, "Delete Watershed File", 1
    MenuSet 2, 6, 1, "Print Current Watershed", 1

    MenuSet 3, 0, 1, "Rainfall", 1
    MenuSet 3, 1, 1, "Create RainFall File", 1
    MenuSet 3, 2, 1, "Edit/View Rainfall File", 1
    MenuSet 3, 3, 1, "Retrieve Rainfall File", 1
    MenuSet 3, 4, 1, "Save Rainfall File", 1
    MenuSet 3, 5, 1, "Delete a Rainfall File", 1
    MenuSet 3, 6, 1, "Use Dimensionless Curves", 1
    MenuSet 3, 7, 1, "Print Rainfall File", 1

    MenuSet 4, 0, 1, "Hydrograph", 1
    MenuSet 4, 1, 1, "Generate Hydrograph", 7
    MenuSet 4, 2, 1, "View Hydrograph", 1
    MenuSet 4, 3, 1, "Retrieve Hydrograph", 1
    MenuSet 4, 4, 1, "Save Hydrograph", 1
    MenuSet 4, 5, 1, "Plot Hydrograph", 3
    MenuSet 4, 6, 1, "Print Hydrograph", 1

    MenuSet 5, 0, 1, "Pond", 1
    MenuSet 5, 1, 1, "Create Pond", 1
    MenuSet 5, 2, 1, "Edit Stage Storage Discharge", 1
    MenuSet 5, 3, 1, "Retrieve Pond", 1
    MenuSet 5, 4, 1, "Save Pond", 1

    MenuSet 6, 0, 1, "Routing", 4
    MenuSet 6, 1, 1, "Define Node and Type", 1
    MenuSet 6, 2, 1, "Link Defined Nodes", 1
    MenuSet 6, 3, 1, "Output Batch Data File", 8
    MenuSet 6, 4, 1, "Display Defined Nodes", 3
    MenuSet 6, 5, 1, "Plot Defined Nodes", 3
    MenuSet 6, 6, 1, "Print Nodal Analysis", 1
    MenuSet 6, 7, 1, "View Active Node", 1
    MenuSet 6, 8, 1, "Unlink a Node", 1

    MenuSet 7, 0, 1, "Order", 1
    MenuSet 7, 1, 1, "Print Order Form", 1

    SHortCutKeySet 1, 1, CHR$(0) + CHR$(59) ' F1
    SHortCutKeySet 1, 2, CHR$(0) + CHR$(60) ' F2
    SHortCutKeySet 1, 3, CHR$(0) + CHR$(61) ' F3

    SHortCutKeySet 1, 6, CHR$(24)          ' Ctrl-X

    IF NOT DisplayType THEN
        MenuColor 0, 7, 15, 8, 7, 0, 15   'Best for monochrome and colors
    ELSE
        MenuColor 0, 7, 4, 8, 15, 0, 12   'Best for color
    END IF

    MenuPreProcess

END SUB

DEFSNG A-Z
SUB ShedCalc (w AS watershedtype)

IF w.DCIA <> 0 THEN w.FracDCIA = (w.ImpArea / TotalArea) * (w.DCIA / 100)
IF TotalArea <> 0 THEN
        w.FracNDCIA = (w.ImpArea / TotalArea) * ((100 - w.DCIA) / 100)
        w.FracPerv = (TotalArea - w.ImpArea) / TotalArea
END IF
END SUB

DEFINT A-Y
SUB UnlinkNode
ActiveNode = GetNode("Unlink what Node?", FALSE, FALSE)
IF ActiveNode = 0 OR FlowTo(ActiveNode) = 0 THEN EXIT SUB
FOR i = 1 TO MAXSTEPS: NodeInflow(FlowTo(ActiveNode), i) = NodeInflow(FlowTo(ActiveNode), i) - NodeOutflow(ActiveNode, i): NodeOutflow(ActiveNode, i) = 0: NEXT i
FlowTo(ActiveNode) = 0
END SUB

SUB ViewHydrograph (NodeFlag, Steps, p() AS pondtype)

IF HydroSteps = 0 THEN
        a$ = "No Hydrograph is currently active in memory, either retrieve|"
        a$ = a$ + "a hydrograph from disk or generate one."
        junk = Alert(4, a$, 9, 10, 14, 70, "", "", "")
        EXIT SUB
END IF
IF NodeFlag = (TRUE) THEN
        ActiveNode = GetNode("Choose Node to View", FALSE, FALSE)
        IF ActiveNode = 0 THEN EXIT SUB
END IF

WindowOpen 1, 3, 2, 23, 75, 14, 1, 14, 1, 15, FALSE, TRUE, FALSE, TRUE, 2, "HYDROGRAPH"
IF NodeFlag = FALSE THEN DrawHydrograph 0
IF NodeFlag = TRUE THEN DrawNode 0, p()
MenuOff
ButtonOpen 1, 1, "", 4, 72, 18, 72, 6
Position = 1
ExitFlag = FALSE
currButton = 1
currstate = 1

WHILE NOT ExitFlag
WindowDo currButton, 0
x = Dialog(0)
Position = (currstate - 1) * 8

SELECT CASE x
        CASE 1
                Scrollcode = Dialog(19)
        CASE 4, 9
                WindowClose 1
                MenuOn
                EXIT SUB
        CASE 10
                Scrollcode = -1
        CASE 11
                Scrollcode = -2
        CASE ELSE
                Scrollcode = Dialog(19)
        END SELECT


SELECT CASE Scrollcode
        CASE -1
                IF currstate > 1 THEN currstate = currstate - 1
        CASE -2
                IF currstate < 13 THEN currstate = currstate + 1
        CASE ELSE
                currstate = Scrollcode
END SELECT
IF currstate < 1 THEN currstate = 1
Position = (currstate - 1) * 12
IF NodeFlag = FALSE THEN DrawHydrograph Position
IF NodeFlag = TRUE THEN DrawNode Position, p()
ButtonSetState 1, currstate
WEND
MenuOn
END SUB

